--Universitat Oberta de Catalunya
--TFC: Bases de Dades Relacionals
--Disseny i implementaci del datawarehose d'una cadena de roba
--Autor: Marta Soler
--Data: Octubre 2015

----------------------------------------------------------------------
--SCRIPT DE CREACI DELS OBJECTES DE LA BASE DE DADES               --
----------------------------------------------------------------------

--Creem la seqncia per a la taula PROC_LOG
CREATE SEQUENCE PROC_LOG_SEQ
	START WITH 1
	INCREMENT BY 1
	NOCYCLE
	NOCACHE;
	
--Taula AUX_ERR_MISSATGES: recull els codi d'error i els seus missatges.
--s una taula auxiliar que ens permet personalitazar els missatges d'error.
CREATE TABLE AUX_ERR_MISSATGES
  (
    idMissatge	INT,
    valor		VARCHAR2(150),
    CONSTRAINT AUX_ERR_MISSATGES_PK PRIMARY KEY (idMissatge)
  );

--Taula BOTIGA: recull les dades referent a les botigues
CREATE TABLE Botiga
  (
    idBotiga        	INT NOT NULL ,
    mailResponsable 	VARCHAR2 (100) NOT NULL ,
    numTreballadors 	INT NOT NULL ,
    esFranquicia		NUMBER(1,0) NOT NULL,  --1 si s franqucia, 0 si no ho s
    esVirtual       	NUMBER(1,0) NOT NULL,  --1 si s virtual, 0 si s fsica
    dataBaixa       	DATE ,
    idCiutat 			INT NOT NULL,
    CONSTRAINT Botiga_PK PRIMARY KEY ( idBotiga )
  ) ;

--Taula CIUTAT: recull les dades referents a les ciutats
CREATE TABLE Ciutat
  (
    idCiutat    INT NOT NULL ,
    nom         VARCHAR2(100) NOT NULL ,
    idRegio 	INT NOT NULL,
    CONSTRAINT Ciutat_PK PRIMARY KEY ( idCiutat )
  ) ;

--Taula CIUTAT: recull estadístiques referents a les vendes de cada any i també històriques
CREATE TABLE Estadistica
  (
    idAny                  NUMBER (4,0) NOT NULL ,
    sumBenefici            NUMBER (38,2),
    idBotigaMaxBenefici    INT,
    sumBotigaMaxBenefici   NUMBER (38,2),
    idProducteMaxVendes    NUMBER (13,0),
    sumProducteMaxVendes   INT,
    idHoraMaxVendes        NUMBER (2),
    sumHoraMaxVendes       INT,
    idHoraMinVendes        NUMBER (2) ,
    sumHoraMinVendes       INT ,
    idDiaMaxVendes         NUMBER (2),
    sumDiaMaxVendes        INT ,
    idDiaMinVendes         NUMBER (2),
    sumDiaMinVendes        INT ,
    idCiutatMaxBenefici    INT,
    sumCiutatMaxBenefici   NUMBER (38,2),
    percentBeneficiVirtual NUMBER (5,2),
	CONSTRAINT Estadistica_PK PRIMARY KEY ( idAny )
  ) ;

--Taula EST_BOTIGA: emmagatezema les estadstiques parcials de botiga.
  CREATE TABLE EST_BOTIGA 
  (
	idAny NUMBER(4,0) NOT NULL, 
	idBotiga INT NOT NULL, 
	beneficiBotiga NUMBER NOT NULL,
	CONSTRAINT EST_BOTIGA_PK PRIMARY KEY ( idAny, idBotiga )
   );
   
--Taula EST_CIUTAT: emmagatezema les estadstiques parcials de ciutat.
  CREATE TABLE EST_CIUTAT 
  (
	idAny NUMBER(4,0) NOT NULL, 
	idCiutat INT NOT NULL, 
	beneficis NUMBER NOT NULL,
	CONSTRAINT EST_CIUTAT_PK PRIMARY KEY ( idAny, idCiutat )
   );
   
--Taula EST_DIA: emmagatezema les estadstiques parcials pels dies del mes.
  CREATE TABLE EST_DIA 
  (
	idAny NUMBER(4,0) NOT NULL, 
	idDia INT NOT NULL, 
	n_prod_venuts INT NOT NULL,
	CONSTRAINT EST_DIA_PK PRIMARY KEY ( idAny, idDia )
   );
   
--Taula EST_HORA: emmagatezema les estadstiques parcials per les hores deldia.
  CREATE TABLE EST_HORA 
  (
	idAny NUMBER(4,0) NOT NULL, 
	idHora INT NOT NULL, 
	n_prod_venuts INT NOT NULL,
	CONSTRAINT EST_HORA_PK PRIMARY KEY ( idAny, idHora )
   );
   
--Taula EST_PRODUCTE: emmagatezema les estadstiques parcials de producte.
  CREATE TABLE EST_PRODUCTE 
  (
	idAny NUMBER(4,0) NOT NULL, 
	idProducte INT NOT NULL, 
	numVenuts INT NOT NULL,
	CONSTRAINT EST_PRODUCTE_PK PRIMARY KEY ( idAny, idProducte )
   );
   
 --Taula EST_VIRTUAL: emmagatezema les estadstiques parcials per les vendes virtuals i totals.
  CREATE TABLE EST_VIRTUAL 
  (
	idAny NUMBER(4,0) NOT NULL, 
	beneficiVirtual NUMBER NOT NULL, 
	beneficiTotal NUMBER NOT NULL,
	CONSTRAINT EST_VIRTUAL_PK PRIMARY KEY ( idAny )
   );



--Taula PROC_LOG: emmagatzema un LOG de les crides a procediments emmagatzemats i els seus resultats
CREATE TABLE EXEC_LOG
  (
    idLOG       INT NOT NULL ,
    execTime    TIMESTAMP NOT NULL ,
    nomProc		VARCHAR2(30) NOT NULL,
	inParams	VARCHAR2(500),
	resExec		INT NOT NULL, -- 0 si s correcte, sin codi error.
	CONSTRAINT LOG_PK PRIMARY KEY ( idLOG )
  ) ;


--Taula PRODUCTE: recull les dades referents als productes del catleg
CREATE TABLE Producte
  (
    idProducte       NUMBER (13,0) NOT NULL ,
    descripcio       VARCHAR2 (300) NOT NULL ,
    dataIncorporacio DATE NOT NULL ,
    dataBaixa        DATE,
	CONSTRAINT Producte_PK PRIMARY KEY ( idProducte )
  ) ;


--Taula REGIO: recull les dades referents a les regions on es poden trobar les botigues
CREATE TABLE Regio
  ( 
    idRegio INT NOT NULL ,
    nom 	VARCHAR2 (100) NOT NULL,
	CONSTRAINT Regio_PK PRIMARY KEY ( idRegio )
  ) ;


--Taula VENTA: recull esl fets referents a les ventes
CREATE TABLE VENTA
  (
    idBotiga      INT NOT NULL ,
    idProducte    NUMBER (13,0) NOT NULL ,
    dataVenta     DATE NOT NULL ,
    horaVenta     NUMBER (2,0) NOT NULL ,
    sumProducte   INT NOT NULL ,
    preuBrutTotal NUMBER (38,2) NOT NULL ,
    beneficiTotal NUMBER (38,2) NOT NULL,
    CONSTRAINT VENTA_PK PRIMARY KEY (idBotiga, idProducte, dataVenta, horaVenta),
    CONSTRAINT VENTA_HORA_CK CHECK (horaVenta>=0 AND horaVenta<=23)
  ) ;

-- Definici de claus fornies
ALTER TABLE Botiga ADD CONSTRAINT Botiga_Ciutat_FK FOREIGN KEY ( idCiutat ) REFERENCES Ciutat ( idCiutat ) ;

ALTER TABLE Ciutat ADD CONSTRAINT Ciutat_Regio_FK FOREIGN KEY ( idRegio ) REFERENCES Regio ( idRegio ) ;

ALTER TABLE Estadistica ADD CONSTRAINT EST_BotigaMaxBenefici_FK FOREIGN KEY ( idBotigaMaxBenefici ) REFERENCES Botiga ( idBotiga ) ;

ALTER TABLE Estadistica ADD CONSTRAINT EST_CiutatMaxBenefici_FK FOREIGN KEY ( idCiutatMaxBenefici ) REFERENCES Ciutat ( idCiutat ) ;

ALTER TABLE Estadistica ADD CONSTRAINT EST_ProducteMaxVendes_FK FOREIGN KEY ( idProducteMaxVendes ) REFERENCES Producte ( idProducte ) ;

ALTER TABLE VENTA ADD CONSTRAINT VENTA_Botiga_FK FOREIGN KEY ( idBotiga ) REFERENCES Botiga ( idBotiga ) ;

ALTER TABLE VENTA ADD CONSTRAINT VENTA_Producte_FK FOREIGN KEY ( idProducte ) REFERENCES Producte ( idProducte ) ;


--Inicialitzaci de la taula AUX_ERR_MISSATGES amb els seus valors
Insert into AUX_ERR_MISSATGES values ('101001','ERROR ALTA BOTIGA: Botiga ja existeix.');
Insert into AUX_ERR_MISSATGES values ('101002','ERROR ALTA BOTIGA: Ciutat no existeix.');
Insert into AUX_ERR_MISSATGES values ('101003','ERROR ALTA BOTIGA: Tots els camps sn obligatoris. No s''ha informat un dels camps.');
Insert into AUX_ERR_MISSATGES values ('101004','ERROR ALTA BOTIGA: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('102001','ERROR MODIFICACI BOTIGA: Botiga no existeix.');
Insert into AUX_ERR_MISSATGES values ('102002','ERROR MODIFICACI BOTIGA: Botiga donada de baixa.');
Insert into AUX_ERR_MISSATGES values ('102003','ERROR MODIFICACI BOTIGA: Ciutat no existeix.');
Insert into AUX_ERR_MISSATGES values ('102004','ERROR MODIFICACI BOTIGA: Tots els camps sn obligatoris. No s''ha informat un dels camps.');
Insert into AUX_ERR_MISSATGES values ('103001','ERROR BAIXA BOTIGA: Botiga no existeix.');
Insert into AUX_ERR_MISSATGES values ('103002','ERROR BAIXA BOTIGA: Botiga ja donada de baixa anteriorment.');
Insert into AUX_ERR_MISSATGES values ('103003','ERROR BAIXA BOTIGA: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('104001','ERROR ALTA PRODUCTE: Producte ja existeix.');
Insert into AUX_ERR_MISSATGES values ('104002','ERROR ALTA PRODUCTE: Tots els camps sn obligatoris. No s''ha informat un dels camps.');
Insert into AUX_ERR_MISSATGES values ('104003','ERROR ALTA PRODUCTE: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('105001','ERROR MODIFICACI PRODUCTE: Producte no existeix.');
Insert into AUX_ERR_MISSATGES values ('105002','ERROR MODIFICACI PRODUCTE: Producte donat de baixa.');
Insert into AUX_ERR_MISSATGES values ('105003','ERROR MODIFICACI PRODUCTE: Tots els camps sn obligators. No s''ha informat algn dels camps.');
Insert into AUX_ERR_MISSATGES values ('106001','ERROR BAIXA PRODUCTE: El producte no existeix.');
Insert into AUX_ERR_MISSATGES values ('106002','ERROR BAIXA PRODUCTE: El producte ja donat de baixa anteriorment.');
Insert into AUX_ERR_MISSATGES values ('106003','ERROR BAIXA PRODUCTE: Altres errors');
Insert into AUX_ERR_MISSATGES values ('107001','ERROR ALTA VENTA: Venta ja existeix.');
Insert into AUX_ERR_MISSATGES values ('107002','ERROR ALTA VENTA: Botiga no existeix.');
Insert into AUX_ERR_MISSATGES values ('107003','ERROR ALTA VENTA: Producte no existeix.');
Insert into AUX_ERR_MISSATGES values ('107004','ERROR ALTA VENTA: Format hora incorrecte (s''ha d''especificar un nombre entre 0 i 23)');
Insert into AUX_ERR_MISSATGES values ('107005','ERROR ALTA VENTA: Botiga donada de baixa.');
Insert into AUX_ERR_MISSATGES values ('107006','ERROR ALTA VENTA: Producte fora de catleg');
Insert into AUX_ERR_MISSATGES values ('107007','ERROR ALTA VENTA: Tots els camps sn obligatoris. No s''ha informat algn camp.');
Insert into AUX_ERR_MISSATGES values ('107008','ERROR ALTA VENTA: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('108001','ERROR MODIFICACI VENTA: Venta no existeix.');
Insert into AUX_ERR_MISSATGES values ('108002','ERROR MODIFICACI VENTA: Tots els camps sn obligatoris. No s''ha informat algn camp.');
Insert into AUX_ERR_MISSATGES values ('108003','ERROR MODIFICACI VENTA: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('109001','ERROR BAIXA VENTA: Venta no existeix.');
Insert into AUX_ERR_MISSATGES values ('109002','ERROR BAIXA VENTA: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('105004','ERROR MODIFICACI PRODUCTE: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('102005','ERROR MODIFICACI BOTIGA: Altres errors.');
Insert into AUX_ERR_MISSATGES values ('110001','ERROR RESUM MENSUAL BOTIGA: Format d''any incorrecte. S''ha d''especificar els 4 dgits de l''any.');
Insert into AUX_ERR_MISSATGES values ('110002','ERROR RESUM MENSUAL BOTIGA: Format de mes incorrecte. S''ha d''especificar un nombre entre 1 i 12.');
Insert into AUX_ERR_MISSATGES values ('110003','ERROR RESUM MENSUAL BOTIGA: No hi ha dades disponibles per les dates especificades.');
Insert into AUX_ERR_MISSATGES values ('110004','ERROR RESUM MENSUAL BOTIGA: Altres Errors');
Insert into AUX_ERR_MISSATGES values ('111001','ERROR RESUM MENSUAL PRODUCTE: Format d''any incorrecte. S''ha d''especificar els 4 dgits de l''any.');
Insert into AUX_ERR_MISSATGES values ('111002','ERROR RESUM MENSUAL PRODUCTE: Format de mes incorrecte. S''ha d''especificar un numbre entre 1 i 12.');
Insert into AUX_ERR_MISSATGES values ('111003','ERROR RESUM MENSUAL PRODUCTE: No hi ha dades disponibles per les dates especificades.');
Insert into AUX_ERR_MISSATGES values ('111004','ERROR RESUM MENSUAL PRODUCTE: Altres Errors.');
Insert into AUX_ERR_MISSATGES values ('112001','ERROR RESUM MENSUAL PER DIES: Format d''any incorrecte. S''ha d''especificar els 4 dgits de l''any.');
Insert into AUX_ERR_MISSATGES values ('112002','ERROR RESUM MENSUAL PER DIES: Format de mes incorrecte. S''ha d''especificar un numbre entre 1 i 12.');
Insert into AUX_ERR_MISSATGES values ('112003','ERROR RESUM MENSUAL PER DIES: No hi ha dades disponibles per les dates especificades.');
Insert into AUX_ERR_MISSATGES values ('112004','ERROR RESUM MENSUAL PER DIES: Altres Errors.');
Insert into AUX_ERR_MISSATGES values ('113001','ERROR ESTADSTICA ANUAL: Format d''any incorrecte. S''ha d''especificar els 4 dgits de l''any.');
Insert into AUX_ERR_MISSATGES values ('113002','ERROR ESTADSTICA ANUAL: No hi ha dades disponibles per a l''any especificat.');
Insert into AUX_ERR_MISSATGES values ('113003','ERROR ESTADSTICA ANUAL: Altres Errors.');
Insert into AUX_ERR_MISSATGES values ('114001','ERROR ESTADSTICA HISTRICA: No hi ha dades disponibles per a realitzar l''estadstica.');
Insert into AUX_ERR_MISSATGES values ('114002','ERROR ESTADSTICA HISTRICA: Altres Errors.');