/**

**/

CREATE OR REPLACE PACKAGE BOTIGA_PKG AS

  --Dna d'alta una nova botiga al sistema
	PROCEDURE ALTA_BOTIGA (p_idBotiga IN Botiga.idBotiga%TYPE, p_idCiutat IN Botiga.idCiutat%TYPE, 
  p_email IN Botiga.mailResponsable%TYPE, p_nTreballadors IN Botiga.numTreballadors%TYPE, 
  p_Franquicia IN Botiga.esFranquicia%TYPE, p_Virtual IN Botiga.esVirtual%TYPE,
  p_RES OUT int, p_ERRDESC OUT VARCHAR2);
  
  --Modifica una botiga ja existent al sistema
	PROCEDURE MODIFICAR_BOTIGA (p_idBotiga IN Botiga.idBotiga%TYPE, p_idCiutat IN Botiga.idCiutat%TYPE, 
  p_email IN Botiga.mailResponsable%TYPE, p_nTreballadors IN Botiga.numTreballadors%TYPE, 
  p_Franquicia IN Botiga.esFranquicia%TYPE, p_Virtual IN Botiga.esVirtual%TYPE,
  p_RES OUT int, p_ERRDESC OUT VARCHAR2);
  
  --Dona de baixa lgica una botiga existent (informa data de baixa)
  PROCEDURE BAIXA_BOTIGA (p_idBotiga IN Botiga.idBotiga%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2);
  
  --Retorna true si la botiga est donada d'alta al sistema
  FUNCTION EXISTEIX_BOTIGA (p_idBotiga Botiga.idBotiga%TYPE) RETURN BOOLEAN;
  
  --Retorna true si la botiga te informada data de baixa
  FUNCTION BOTIGA_ES_BAIXA (p_idBotiga Botiga.idBotiga%TYPE) RETURN BOOLEAN;

  --Retorna true si la botiga ha estat donada de baixa amb data anterior a l'indicada
  FUNCTION BOTIGA_ES_BAIXA (p_idBotiga Botiga.idBotiga%TYPE, p_data DATE) RETURN BOOLEAN;
  
END BOTIGA_PKG;
/

CREATE OR REPLACE PACKAGE BODY BOTIGA_PKG AS
  
  /******************************************
  **           ALTA_BOTIGA                 **
  ******************************************/
  
  PROCEDURE ALTA_BOTIGA (p_idBotiga IN Botiga.idBotiga%TYPE, p_idCiutat IN Botiga.idCiutat%TYPE, 
  p_email IN Botiga.mailResponsable%TYPE, p_nTreballadors IN Botiga.numTreballadors%TYPE,
  p_Franquicia IN Botiga.esFranquicia%TYPE, p_Virtual IN Botiga.esVirtual%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS
	
	--Definir variables
	nomProcediment VARCHAR2(30);
	paramsProcediment VARCHAR2(500);
	logREG INT;
	CAMPS_NO_INFORMATS EXCEPTION;
	CIUTAT_NO_EXISTEIX EXCEPTION;
	PRAGMA exception_INIT(CIUTAT_NO_EXISTEIX, -2291);
	
  BEGIN
	
	--Assignem valors a les variables de LOG
    nomProcediment:='ALTA_BOTIGA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idBotiga||' P2:'||p_idCiutat||' P3:'||p_email||' P4:'||p_nTreballadors||
    ' P5:'||p_Franquicia||' P6:'||p_Virtual;
	
	--Mirem si algn dels camps no est informat
	IF ((p_idBotiga IS NULL) OR (p_idCiutat IS NULL) OR (p_email IS NULL) OR (p_nTreballadors IS NULL) OR (p_Franquicia IS NULL) OR (p_Virtual IS NULL)) THEN
		RAISE CAMPS_NO_INFORMATS;
	END IF;
    
    --Inserim els valors dels parmetres a la taula 
    INSERT INTO BOTIGA (idBotiga, mailResponsable, numtreballadors, esFranquicia, esVirtual,IDCIUTAT)
      VALUES (p_idBotiga, p_email, p_nTreballadors, p_Franquicia, p_Virtual, p_idCiutat);
    
	--Si s'ha creat correctament la botiga afegim el LOG corresponent i confirmem.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('BOTIGA:'||p_idBotiga||' AFEGIDA AMB XIT');
  
  EXCEPTION
    
	--Si la botiga ja existeix a la BDD
    WHEN DUP_VAL_ON_INDEX THEN
		P_RES:=101001;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	
	--Si l'identificador de ciutat indicat no existeix al sistema
	WHEN CIUTAT_NO_EXISTEIX THEN
		P_RES:=101002;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	
	--Si no hem informat algun camp obligatori
	WHEN CAMPS_NO_INFORMATS THEN
		P_RES:=101003;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	
	--Si es dona qualsevol altre excepci
	WHEN OTHERS THEN
		P_RES:=101004;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
		--Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
		dbms_output.put_line(SQLCODE||': '||SQLERRM); 

  END;
  
  
  
  /******************************************
  **       MODIFICAR_BOTIGA                **
  ******************************************/
  
  PROCEDURE MODIFICAR_BOTIGA (p_idBotiga IN Botiga.idBotiga%TYPE, p_idCiutat IN Botiga.idCiutat%TYPE, 
  p_email IN Botiga.mailResponsable%TYPE, p_nTreballadors IN Botiga.numTreballadors%TYPE, 
  p_Franquicia IN Botiga.esFranquicia%TYPE, p_Virtual IN Botiga.esVirtual%TYPE,
  p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS
  
	--Definir variables
	nomProcediment VARCHAR2(30);
	paramsProcediment VARCHAR2(500);
	logREG INT;
	BOTIGA_INEXISTENT EXCEPTION;
	BOTIGA_BAIXA EXCEPTION;
	CAMPS_NO_INFORMATS EXCEPTION;
	CIUTAT_NO_EXISTEIX EXCEPTION;
	PRAGMA exception_INIT(CIUTAT_NO_EXISTEIX, -2291);
  
  BEGIN
	
	--Assignem valors a les variables de LOG
    nomProcediment:='MODIFICAR_BOTIGA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idBotiga||' P2:'||p_idCiutat||' P3:'||p_email||' P4:'||p_nTreballadors||
    ' P5:'||p_Franquicia||' P6:'||p_Virtual;
	
	--Mirem si algn dels camps no est informat
	IF ((p_idBotiga IS NULL) OR (p_idCiutat IS NULL) OR (p_email IS NULL) OR (p_nTreballadors IS NULL) OR (p_Franquicia IS NULL) OR (p_Virtual IS NULL)) THEN
		RAISE CAMPS_NO_INFORMATS;
	END IF;
    
    --Mirem si la botiga no existeix o si es baixa
    IF EXISTEIX_BOTIGA(p_idBotiga)=FALSE THEN
      RAISE BOTIGA_INEXISTENT;
    ELSE IF BOTIGA_ES_BAIXA(p_idBotiga)=TRUE THEN
      RAISE BOTIGA_BAIXA;
      END IF;
    END IF;
    
    --Modifiquem els valors dels parmtres a la taula
    UPDATE BOTIGA SET mailResponsable=p_email, numtreballadors=p_nTreballadors, esFranquicia=p_Franquicia,
      esVirtual=p_Virtual, idCiutat=p_idCiutat WHERE idBotiga=p_idBotiga;
	  
    --Si s'ha actualitzat correctament la botiga afegim el LOG corresponent, confirmem la transacci i mostrem el resultat.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('BOTIGA:'||p_idBotiga||' MODIFICADA AMB XIT');
	
  EXCEPTION
    
	--Si la botiga no existeix a la BDD
    WHEN BOTIGA_INEXISTENT THEN
      P_RES:=102001;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);

	--Si la botiga est donada de baixa
	WHEN BOTIGA_BAIXA THEN
      P_RES:=102002;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	
	--Si l'identificador de ciutat indicat no existeix al sistema
	WHEN CIUTAT_NO_EXISTEIX THEN
		P_RES:=102003;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	
	--Si no hem informat algun camp obligatori
	WHEN CAMPS_NO_INFORMATS THEN
		P_RES:=102004;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	
	--Si es dona qualsevol altre excepci
	WHEN OTHERS THEN
		P_RES:=102005;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
		--Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
		dbms_output.put_line(SQLCODE||': '||SQLERRM); 

  END;


  /******************************************
  **           BAIXA_BOTIGA                **
  ******************************************/
  
  PROCEDURE BAIXA_BOTIGA (p_idBotiga IN Botiga.idBotiga%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS
	
	--Definir variables
	nomProcediment VARCHAR2(30);
	paramsProcediment VARCHAR2(500);
	logREG INT;
	BOTIGA_INEXISTENT EXCEPTION;
	BOTIGA_BAIXA EXCEPTION;
	
  BEGIN
    
	--Assignem valors a les variables de LOG
    nomProcediment:='BAIXA_BOTIGA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idBotiga;
  
	--Mirem si la botiga no existeix o si ja s baixa.
    IF EXISTEIX_BOTIGA(p_idBotiga)=FALSE THEN
      RAISE BOTIGA_INEXISTENT;
    ELSE IF BOTIGA_ES_BAIXA(p_idBotiga)=TRUE THEN
      RAISE BOTIGA_BAIXA;
      END IF;
    END IF;
    
	--Assignem la data del sistema com a data de baixa
    UPDATE BOTIGA SET dataBaixa=SYSDATE WHERE idBotiga=p_idBotiga;
	
	--Si tot s correcte confirmem la transacci, fem el registre al LOG i mostrem el resultat per pantalla.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('BOTIGA:'||p_idBotiga||' DONADA DE BAIXA AMB XIT');
    
    EXCEPTION
      
	  --Si la botiga no existeix
	  WHEN BOTIGA_INEXISTENT THEN
        p_RES:=103001;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      
	  --Si la botiga ja est donada de baixa
	  WHEN BOTIGA_BAIXA THEN
        p_RES:=103002;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      
	  --Si es dona qualsevol altre excepci
	  WHEN OTHERS THEN
        p_RES:=103003;
		EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        dbms_output.put_line('ERROR:'||P_RES||'-'||p_ERRDESC); 
  END;

  
  

  /******************************************
  **        EXISTEIX_BOTIGA                **
  ******************************************/ 
  
  FUNCTION EXISTEIX_BOTIGA (p_idBotiga Botiga.idBotiga%TYPE) RETURN BOOLEAN IS
    ccount int; 
  BEGIN
    SELECT COUNT(*) INTO ccount FROM Botiga WHERE idBotiga=p_idBotiga;
    IF ccount = 0 THEN
      RETURN FALSE;
    ELSE
      RETURN TRUE;
    END IF;
  END;
  
  
  -------------------------------------------
  --          BOTIGA_ES_BAIXA              --
  -------------------------------------------   
  FUNCTION BOTIGA_ES_BAIXA (p_idBotiga Botiga.idBotiga%TYPE) RETURN BOOLEAN IS
    existeix BOOLEAN;
    aux Botiga.idBotiga%TYPE;
  BEGIN
    existeix:=TRUE;
    SELECT idBotiga INTO aux FROM Botiga WHERE idBotiga=p_idBotiga and dataBaixa IS NOT NULL;
    RETURN existeix;
      --Si el select retorna 0 columnes salta un error -01403
  EXCEPTION
    WHEN OTHERS THEN
      existeix:=FALSE;
      RETURN existeix;
  END;
 
 
  -------------------------------------------
  --          BOTIGA_ES_BAIXA            --
  ------------------------------------------- 
  FUNCTION BOTIGA_ES_BAIXA (p_idBotiga Botiga.idBotiga%TYPE, p_data DATE) RETURN BOOLEAN IS
    dBaixa DATE;
    esBaixa BOOLEAN;
  BEGIN
    SELECT dataBaixa INTO dBaixa FROM Botiga WHERE idBotiga=p_idBotiga;
    IF (dBaixa IS NULL) OR (p_data < dBaixa) THEN
      esBaixa:=FALSE;
    ELSE
      esBaixa:=TRUE;
    END IF;
    RETURN esBaixa;
  EXCEPTION
    --Si la consulta no ha retornat cap resultat considerarem que s baixa
    WHEN OTHERS THEN
      esBaixa:=TRUE;
      RETURN esBaixa;
  END; 
END BOTIGA_PKG;