/**

**/

CREATE OR REPLACE PACKAGE PRODUCTE_PKG AS
  
  --Dna d'alta un nou producte a la BDD
	PROCEDURE ALTA_PRODUCTE (p_idProducte IN Producte.idProducte%TYPE, p_descripcio IN Producte.descripcio%TYPE, 
  p_dataInc IN Producte.dataIncorporacio%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2);
  
  --Modifica un producte ja existent a la BDD
	PROCEDURE MODIFICAR_PRODUCTE (p_idProducte IN Producte.idProducte%TYPE, p_descripcio IN Producte.descripcio%TYPE, 
  p_dataInc IN Producte.dataIncorporacio%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2);
  
  --Dona de baixa lgica un producte existent (informa data de baixa)
  PROCEDURE BAIXA_PRODUCTE (p_idProducte IN Producte.idProducte%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2);
  
  --Retorna True si el producte est donat d'alta a la BDD
  FUNCTION EXISTEIX_PRODUCTE (p_idProducte Producte.idProducte%TYPE) RETURN BOOLEAN;
  
  --Retorna True si el producte no existeix o t una data de baixa informada
  FUNCTION PRODUCTE_ES_BAIXA (p_idProducte Producte.idProducte%TYPE) RETURN BOOLEAN;
  
  --Retorna true si el producte no existeix o si a la data especificada est fora de catleg
  FUNCTION PRODUCTE_ES_BAIXA (p_idProducte Producte.idProducte%TYPE, p_data DATE) RETURN BOOLEAN;
  
  --Donat un codi EAN13 d'un producte en retorna la descripci
  FUNCTION GET_DESC_PRODUCTE (p_idProducte Producte.idProducte%TYPE) RETURN Producte.descripcio%TYPE;
  
END PRODUCTE_PKG;
/

CREATE OR REPLACE PACKAGE BODY PRODUCTE_PKG AS
  
  /******************************************
  **           ALTA_PRODUCTE               **
  ******************************************/
  
  PROCEDURE ALTA_PRODUCTE (p_idProducte IN Producte.idProducte%TYPE, p_descripcio IN Producte.descripcio%TYPE, 
  p_dataInc IN Producte.dataIncorporacio%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS

    --Definim variables
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    CAMPS_NO_INFORMATS EXCEPTION;
    --PRAGMA exception_INIT(CAMPS_NO_INFORMATS, -1400);

  BEGIN

  	--Assignem valors a les variables de LOG
    nomProcediment:='ALTA_PRODUCTE';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idProducte||' P2:'||p_descripcio||' P3:'||p_dataInc;
    
    IF ((p_idProducte IS NULL) OR (p_descripcio IS NULL) OR (p_dataInc IS NULL)) THEN
      RAISE CAMPS_NO_INFORMATS;
    END IF;
    
    --Inserim els valors dels parmtres a la taula 
    INSERT INTO PRODUCTE (idProducte, descripcio, dataIncorporacio)
      VALUES (p_idProducte, p_descripcio, p_dataInc);

    --Si s'ha creat correctament la producte afegim el LOG corresponent i confirmem.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('PRODUCTE:'||p_idProducte||' AFEGIT AMB XIT');

  EXCEPTION
    
    --Si la producte ja existeix a la BDD
    WHEN DUP_VAL_ON_INDEX THEN
      P_RES:=104001;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
    --Si no s'ha informat algun camp obligatori
    WHEN CAMPS_NO_INFORMATS THEN
      P_RES:=104002;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
    --Si salta qualsevol altre excepci  
    WHEN OTHERS THEN
      P_RES:=104003;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      --Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
      dbms_output.put_line(SQLCODE||': '||SQLERRM);  
    
  END;
  
  
  
  /******************************************
  **       MODIFICAR_PRODUCTE              **
  ******************************************/
  
  PROCEDURE MODIFICAR_PRODUCTE (p_idProducte IN Producte.idProducte%TYPE, p_descripcio IN Producte.descripcio%TYPE, 
  p_dataInc IN Producte.dataIncorporacio%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS
  
    --Definim variables
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    PRODUCTE_INEXISTENT EXCEPTION;
    PRODUCTE_BAIXA EXCEPTION;
    CAMPS_NO_INFORMATS EXCEPTION;
    --PRAGMA exception_INIT(CAMPS_NO_INFORMATS, -1400);
 
  BEGIN
  	
    --Assignem valors a les variables de LOG
    nomProcediment:='MODIFICAR_PRODUCTE';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idProducte||' P2:'||p_descripcio||' P3:'||p_dataInc;
    
    IF ((p_idProducte IS NULL) OR (p_descripcio IS NULL) OR (p_dataInc IS NULL)) THEN
      RAISE CAMPS_NO_INFORMATS;
    END IF;
    
    --Mirem si la producte no existeix o si es baixa
    IF EXISTEIX_PRODUCTE(p_idProducte)=FALSE THEN
      RAISE PRODUCTE_INEXISTENT;
    ELSE IF PRODUCTE_ES_BAIXA(p_idProducte)=TRUE THEN
      RAISE PRODUCTE_BAIXA;
      END IF;
    END IF;
    
    --Modifiquem els valors dels parmtres a la taula
    UPDATE PRODUCTE SET descripcio=p_descripcio, dataIncorporacio=p_dataInc WHERE idProducte=p_idProducte;
    
    --Si s'ha actualitzat correctament el producte afegim el LOG corresponent, confirmem la transacci i mostrem el resultat.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('PRODUCTE:'||p_idProducte||' MODIFICAT AMB XIT');
  
  EXCEPTION
    
    --Si la producte no existeix a la BDD
    WHEN PRODUCTE_INEXISTENT THEN
      P_RES:=105001;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
    --Si el producte est donat de baixa
    WHEN PRODUCTE_BAIXA THEN
      P_RES:=105002;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);    
   
    --Si no s'ha informat algun camp obligatori
    WHEN CAMPS_NO_INFORMATS THEN
      P_RES:=105003;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
    --Si salta qualsevol altre excepci  
    WHEN OTHERS THEN
      P_RES:=105004;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      --Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
      dbms_output.put_line(SQLCODE||': '||SQLERRM);
      
  END;



  /******************************************
  **           BAIXA_PRODUCTE              **
  ******************************************/
  
  PROCEDURE BAIXA_PRODUCTE (p_idProducte IN Producte.idProducte%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS
  
    --Definim variables
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    PRODUCTE_INEXISTENT EXCEPTION;
    PRODUCTE_BAIXA EXCEPTION;
  
  BEGIN
    
    --Assignem valors a les variables de LOG
    nomProcediment:='BAIXA_PRODUCTE';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idProducte;
  
    --Mirem si la producte no existeix o si ja s baixa.
    IF EXISTEIX_PRODUCTE(p_idProducte)=FALSE THEN
      RAISE PRODUCTE_INEXISTENT;
    ELSE IF PRODUCTE_ES_BAIXA(p_idProducte)=TRUE THEN
      RAISE PRODUCTE_BAIXA;
      END IF;
    END IF;
    
    --Assignem la data del sistema com a data de baixa
    UPDATE PRODUCTE SET dataBaixa=SYSDATE WHERE idProducte=p_idProducte;
	
    --Si tot s correcte confirmem la transacci, fem el registre al LOG i mostrem el resultat per pantalla.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('PRODUCTE:'||p_idProducte||' DONAT DE BAIXA AMB XIT');
    
    EXCEPTION
      --Si el producte no existeix
      WHEN PRODUCTE_INEXISTENT THEN
        p_RES:=106001;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      
      --Si el producte ja s baixa
      WHEN PRODUCTE_BAIXA THEN
        p_RES:=106002;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        p_ERRDESC:='La Producte ja havia estat donat de baixa amb anterioritat';

      --Si salta qualsevol altre excepci
      WHEN OTHERS THEN
        p_RES:=106003;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        --Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
        dbms_output.put_line(SQLCODE||': '||SQLERRM); 
  END;


  -------------------------------------------
  --        EXISTEIX_PRODUCTE              --
  -------------------------------------------   
  FUNCTION EXISTEIX_PRODUCTE (p_idProducte Producte.idProducte%TYPE) RETURN BOOLEAN IS
    aux Producte.idProducte%TYPE;
    
    ccount int;
    
  BEGIN
    SELECT COUNT(*) INTO ccount FROM Producte WHERE idProducte=p_idProducte;
    IF ccount = 0 THEN
      RETURN FALSE;
    ELSE
      RETURN TRUE;
    END IF;
  END;
  
  
  -------------------------------------------
  --          PRODUCTE_ES_BAIXA            --
  -------------------------------------------   
  FUNCTION PRODUCTE_ES_BAIXA (p_idProducte Producte.idProducte%TYPE) RETURN BOOLEAN IS
    existeix BOOLEAN;
    aux Producte.idProducte%TYPE;
  BEGIN
    existeix:=TRUE;
    SELECT idProducte INTO aux FROM Producte WHERE idProducte=p_idProducte and dataBaixa IS NOT NULL;
    RETURN existeix;
      --Si el select retorna 0 columnes salta un error -01403
  EXCEPTION
    WHEN OTHERS THEN
      existeix:=FALSE;
      RETURN existeix;
  END;
 
 
  -------------------------------------------
  --          PRODUCTE_ES_BAIXA            --
  ------------------------------------------- 
  FUNCTION PRODUCTE_ES_BAIXA (p_idProducte Producte.idProducte%TYPE, p_data DATE) RETURN BOOLEAN IS
    dInc DATE;
    dBaixa DATE;
    esBaixa BOOLEAN;
  BEGIN
    SELECT dataIncorporacio, dataBaixa INTO dInc, dBaixa FROM Producte WHERE idProducte=p_idProducte;
    IF (dBaixa IS NULL) THEN
      IF dInc <= p_data THEN
        esBaixa:=FALSE;
      ELSE
        esBaixa:=TRUE;
      END IF;
    ELSE
      IF p_data BETWEEN dInc AND dBaixa THEN
        esBaixa:=FALSE;
      ELSE
        esBaixa:=TRUE;
      END IF;
    END IF;
    RETURN esBaixa;
  EXCEPTION
    --Si la consulta no ha retornat cap resultat considerarem que s baixa
    WHEN OTHERS THEN
      esBaixa:=TRUE;
      RETURN esBaixa;
  END;
  
  -------------------------------------------
  --         GET_DESC_PRODUCTE             --
  -------------------------------------------   
  FUNCTION GET_DESC_PRODUCTE (p_idProducte Producte.idProducte%TYPE) RETURN Producte.descripcio%TYPE IS
    desc_prod Producte.descripcio%TYPE;
  BEGIN
	SELECT descripcio INTO desc_prod FROM Producte where idProducte=p_idProducte;
	RETURN desc_prod;
  END;
  
END PRODUCTE_PKG;