/**

**/

CREATE OR REPLACE PACKAGE VENTA_PKG AS

	--Dona d'alta una nova venda al sistema
PROCEDURE ALTA_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE, p_sumProd IN Venta.sumProducte%TYPE,
  p_preuBrut IN Venta.preuBrutTotal%TYPE, p_benefici IN Venta.beneficiTotal%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2);

	--Modifica una venda ja existent
PROCEDURE MODIFICAR_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE, p_sumProd IN Venta.sumProducte%TYPE,
  p_preuBrut IN Venta.preuBrutTotal%TYPE, p_benefici IN Venta.beneficiTotal%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2); 

	--Elimina una venda del sistema
PROCEDURE BAIXA_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2);

  --Ens diu si una venda est registrada al sistema
FUNCTION EXISTEIX_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE) RETURN BOOLEAN;
  
END VENTA_PKG;
/

CREATE OR REPLACE PACKAGE BODY VENTA_PKG AS
  
  /******************************************
  **           ALTA_VENTA                  **
  ******************************************/

  PROCEDURE ALTA_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE, p_sumProd IN Venta.sumProducte%TYPE, 
  p_preuBrut IN Venta.preuBrutTotal%TYPE, p_benefici IN Venta.beneficiTotal%Type, p_RES OUT int, 
  p_ERRDESC OUT VARCHAR2) IS
  
	  --Definir variables
	  nomProcediment VARCHAR2(30);
	  paramsProcediment VARCHAR2(500);
	  logREG INT;
	  PRODUCTE_NO_EXISTEIX EXCEPTION;
	  BOTIGA_NO_EXISTEIX EXCEPTION;
	  PRODUCTE_ES_BAIXA EXCEPTION;
	  BOTIGA_ES_BAIXA EXCEPTION;
	  CAMPS_NO_INFORMATS EXCEPTION;
	  CK_EXCEPTION EXCEPTION;
	  PRAGMA exception_INIT(CK_EXCEPTION, -2290);
  
  BEGIN
	--Assignem valors a les variables de LOG
    nomProcediment:='ALTA_VENTA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idBotiga||' P2:'||p_idProducte||' P3:'||p_data||' P4:'||p_hora||
    ' P5:'||p_sumProd||' P6:'||p_preuBrut||' P7:'||p_benefici;

    IF ((p_idBotiga IS NULL) OR (p_idProducte IS NULL) OR (p_data IS NULL) OR (p_hora IS NULL)
		OR (p_sumProd IS NULL) OR (p_PreuBrut IS NULL) OR (p_benefici IS NULL)) THEN
      RAISE CAMPS_NO_INFORMATS;
    END IF;
    
    --Mirem si la botiga i el producte estan donats d'alta
    IF PRODUCTE_PKG.EXISTEIX_PRODUCTE(p_idProducte)=FALSE THEN
      RAISE PRODUCTE_NO_EXISTEIX;
    ELSE IF BOTIGA_PKG.EXISTEIX_BOTIGA(p_idBotiga)=FALSE THEN
      RAISE BOTIGA_NO_EXISTEIX;
    ELSE IF PRODUCTE_PKG.PRODUCTE_ES_BAIXA(p_idProducte,p_data)=TRUE THEN
      RAISE PRODUCTE_ES_BAIXA;
    ELSE IF BOTIGA_PKG.BOTIGA_ES_BAIXA(p_idBotiga,p_data)=TRUE THEN
      RAISE BOTIGA_ES_BAIXA;
    END IF; END IF; END IF; END IF;
    
    --Inserim els valors dels parmtres a la taula 
    INSERT INTO VENTA (idBotiga, idProducte, dataVenta, horaVenta, sumProducte, preuBrutTotal, beneficiTotal)
      VALUES (p_idBotiga, p_idProducte, p_data, p_hora, p_sumProd, p_preuBrut, p_benefici);
    
	--Si s'ha creat correctament la venta afegim el LOG corresponent i confirmem.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('VENTA AFEGIDA AMB XIT');
	
  EXCEPTION
    
	--Si la venta ja existeix a la BDD
    WHEN DUP_VAL_ON_INDEX THEN
      P_RES:=107001;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
	--Si la botiga no existeix
    WHEN BOTIGA_NO_EXISTEIX THEN
      p_RES:=107002;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	  
	--Si el producte no existeix
    WHEN PRODUCTE_NO_EXISTEIX THEN
      p_RES:=107003;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
	--Si la botiga est donada de baixa per aquesta data
    WHEN BOTIGA_ES_BAIXA THEN
      p_RES:=107005;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
	--Si el producte est donat de baixa per aquesta data
    WHEN PRODUCTE_ES_BAIXA THEN
      p_RES:=107006;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
	--Si no s'han informat tots els camps
	WHEN CAMPS_NO_INFORMATS THEN
      P_RES:=107007;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
	--Si l'hora de la venta no s vlida
	WHEN CK_EXCEPTION THEN
      P_RES:=107004;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
	--Si salta qualsevol altre excepci
	WHEN OTHERS THEN
      P_RES:=107008;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      dbms_output.put_line(SQLCODE||': '||SQLERRM);
      
  END;
  
  
  
  /******************************************
  **           MODIFICAR_VENTA             **
  ******************************************/
  
  PROCEDURE MODIFICAR_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE, p_sumProd IN Venta.sumProducte%TYPE, 
  p_preuBrut IN Venta.preuBrutTotal%TYPE, p_benefici IN Venta.beneficiTotal%Type, p_RES OUT int, 
  p_ERRDESC OUT VARCHAR2) IS
  
	  --Definir variables
	  nomProcediment VARCHAR2(30);
	  paramsProcediment VARCHAR2(500);
	  logREG INT;
	  VENTA_NO_EXISTEIX EXCEPTION;
	  CAMPS_NO_INFORMATS EXCEPTION;
  
  BEGIN
	
	--Assignem valors a les variables de LOG
    nomProcediment:='MODIFICAR_VENTA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idBotiga||' P2:'||p_idProducte||' P3:'||p_data||' P4:'||p_hora||
    ' P5:'||p_sumProd||' P6:'||p_preuBrut||' P7:'||p_benefici;
    
	IF ((p_idBotiga IS NULL) OR (p_idProducte IS NULL) OR (p_data IS NULL) OR (p_hora IS NULL)
		OR (p_sumProd IS NULL) OR (p_PreuBrut IS NULL) OR (p_benefici IS NULL)) THEN
      RAISE CAMPS_NO_INFORMATS;
    END IF;
	
	--Mirem si la venta existeix al sistema
    IF EXISTEIX_VENTA(p_idBotiga,p_idProducte,p_data,p_hora)=FALSE THEN
      RAISE VENTA_NO_EXISTEIX;
    END IF;
    
    --Inserim els valors dels parmtres a la taula 
    UPDATE VENTA SET sumProducte=p_sumProd, preuBrutTotal=p_preuBrut, beneficiTotal=p_benefici
      WHERE idBotiga=p_idBotiga AND idProducte=p_idProducte AND dataVenta=p_data and horaVenta=p_hora;
    
	--Si s'ha creat correctament la venta afegim el LOG corresponent i confirmem.
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('VENTA MODIFICADA AMB XIT');
	
  EXCEPTION
    --Si la venta no existeix a la BDD
    WHEN VENTA_NO_EXISTEIX THEN
      P_RES:=108001;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	  
	--Si no s'ha informat algun camps
    WHEN CAMPS_NO_INFORMATS THEN
      P_RES:=108002;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	  
	--Si es dona qualsevol altre excepci
    WHEN OTHERS THEN
      P_RES:=108003;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      dbms_output.put_line('ERROR:'||P_RES||'-'||p_ERRDESC);
      
  END;
  
  
  
  /******************************************
  **            BAIXA_VENTA                **
  ******************************************/
  
  PROCEDURE BAIXA_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE, p_RES OUT int, p_ERRDESC OUT VARCHAR2) IS

	--Declarem les variables
	nomProcediment VARCHAR2(30);
	paramsProcediment VARCHAR2(500);
	logREG INT;
	VENTA_NO_EXISTEIX EXCEPTION;
	
  BEGIN
  
	--Assignem valors a les variables de LOG
    nomProcediment:='BAIXA_VENTA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_idBotiga||' P2:'||p_idProducte||' P3:'||p_data||' P4:'||p_hora;
    
    IF EXISTEIX_VENTA(p_idBotiga,p_idProducte,p_data,p_hora)=FALSE THEN
      RAISE VENTA_NO_EXISTEIX;
    END IF;
    
	--Eliminem la venta de la taula
    DELETE FROM Venta WHERE idBotiga=p_idBotiga AND idProducte=p_idProducte AND dataVenta=p_data AND horaVenta=p_hora;
    
	--Si s'ha eliminat correctament la fila afegim el LOG i confirmem;
	logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
    dbms_output.put_line('VENTA DONADA DE BAIXA AMB XIT');
    
  EXCEPTION
  
	--Si la venta no existeix
    WHEN VENTA_NO_EXISTEIX THEN
      P_RES:=109001;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
	  
	--Si salta qualsevol altra excepci
    WHEN OTHERS THEN
      P_RES:=109002;
	  EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC); 
      dbms_output.put_line('ERROR:'||P_RES||'-'||p_ERRDESC);
	  
  END;

  
  
  /******************************************
  **          EXISTEIX_VENTA               **
  ******************************************/
  
  FUNCTION EXISTEIX_VENTA (p_idBotiga IN Venta.idBotiga%TYPE, p_idProducte IN Venta.idProducte%TYPE, 
  p_data IN Venta.dataVenta%TYPE, p_hora IN Venta.horaVenta%TYPE) RETURN BOOLEAN IS
	ccount int;
  BEGIN
    SELECT COUNT(*) INTO ccount FROM VENTA WHERE idBotiga=p_idBotiga AND idProducte=p_idProducte AND
      dataVenta=p_data AND horaVenta=p_hora;
	IF ccount = 0 THEN
      RETURN FALSE;
    ELSE
      RETURN TRUE;
    END IF;
  END;
  
END VENTA_PKG;