/**

**/

CREATE OR REPLACE PACKAGE CONSULTES_PKG AS
	
	--Procedimet que donat un any i un mes mostra per pantalla un resum mensual
	--de les vendes de cada botiga ordenant les botigues de més a menys beneficis.
	PROCEDURE RESUM_MENSUAL_BOTIGA(p_any IN NUMBER, p_mes IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2 , p_result OUT SYS_REFCURSOR);
	
	--Procedimet que donat un any i un mes mostra per pantalla un resum mensual
	--de les vendes dels productes del catàleg ordenant-lo de més a menys beneficis.
	PROCEDURE RESUM_MENSUAL_PRODUCTE(p_any IN NUMBER, p_mes IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2, p_result OUT SYS_REFCURSOR);
	
	--Procedimet que donat un any i un mes mostra per pantalla un resum mensual
	--de les vendes per dies del mes.
	PROCEDURE RESUM_MENSUAL_PER_DIES(p_any IN NUMBER, p_mes IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2, p_result OUT SYS_REFCURSOR);
  
  --Procediment que mostra per la sortida DBMS el resum mensual per botigues
  PROCEDURE MOSTRAR_RESUM_MENSUAL_BOTIGA(p_any IN NUMBER, p_mes IN NUMBER);

  --Procediment que mostra per la sortida DBMS el resum mensual per productes
  PROCEDURE MOSTRAR_RESUM_MENSUAL_PRODUCTE(p_any IN NUMBER, p_mes IN NUMBER);
  
  --Procediment que mostra per la sortida DBMS el resum mensual per dies
  PROCEDURE MOSTRAR_RESUM_MENSUAL_PER_DIES(p_any IN NUMBER, p_mes IN NUMBER);
  
  --Funci que retorna true si per un determinat ms i any hi ha dades, FALSE si no.
  FUNCTION HI_HA_DADES(p_any NUMBER, p_mes NUMBER) RETURN BOOLEAN;

  --Funci que retorna la botiga que ms unitats ha venut el mes p_mes de l'any p_any del producte p_prod
  FUNCTION BOTIGA_MES_UNITATS (p_any NUMBER, p_mes NUMBER, p_prod NUMBER) RETURN NUMBER;
  
  --Funci que retorna el nombre de productes que ha venut la botiga que ms unitats ha venut el mes p_mes de l'any p_any del producte p_prod
  FUNCTION UN_BOTIGA_MES_UNITATS (p_any NUMBER, p_mes NUMBER, p_prod NUMBER) RETURN NUMBER;
  
  --Funci que retorna el producte ms venut a la cadena per una data p_data
  FUNCTION PRODUCTE_MES_VENUT_DIA (p_data DATE) RETURN NUMBER;
  
  --Funci que retorna el nombre d'unitats venudes del producte ms venut a la cadena per la data p_data
  FUNCTION UN_PRODUCTE_MES_VENUT_DIA (p_data DATE) RETURN NUMBER;
  
  --Funci que retorna la botiga que ha obtinguts ms beneficis per la data p_data
  FUNCTION BOTIGA_MES_BENEFICI (p_data DATE) RETURN NUMBER;
  
  --Funci que retorna el valor dels beneficis que ha obtingut la botiga amb ms beneficis per la data p_data
  FUNCTION VAL_BOTIGA_MES_BENEFICI (p_data DATE) RETURN NUMBER;
	
END CONSULTES_PKG;
/

CREATE OR REPLACE PACKAGE BODY CONSULTES_PKG AS

  /******************************************
  **       RESUM_MENSUAL_BOTIGA            **
  ******************************************/

  PROCEDURE RESUM_MENSUAL_BOTIGA(p_any IN NUMBER, p_mes IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2, p_result OUT SYS_REFCURSOR) IS
    
    --Declarem les variables
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    beneficiCadena NUMBER;
    ANY_INCORRECTE EXCEPTION;
    MES_INCORRECTE EXCEPTION;
    DADES_NO_DISPONIBLES EXCEPTION;
    
  BEGIN
    
    --Assignem valors a les variables de LOG
    nomProcediment:='RESUM_MENSUAL_BOTIGA';
    p_RES:=0;
    paramsProcediment:='P1:'||p_any||' P2:'||p_mes;
    
    --Mirem si el mes i l'any especificats son correctes
    IF p_any NOT BETWEEN 1000 AND 9999 THEN
      RAISE ANY_INCORRECTE;
    END IF;
    IF p_mes NOT BETWEEN 1 AND 12 THEN
      RAISE MES_INCORRECTE;
    END IF;
    
    --Mirem si hi ha dades per aquest perode
    IF HI_HA_DADES(p_any,p_mes)=FALSE THEN
      RAISE DADES_NO_DISPONIBLES;
    END IF;
    
    SELECT sum(beneficiTotal) INTO beneficiCadena FROM VENTA
      WHERE EXTRACT(MONTH FROM dataventa)=p_mes AND
      EXTRACT (YEAR FROM dataventa)=p_any;
    --Obtenim la consulta en un cursor variable
    open p_result for
      SELECT idBotiga,
        sum(sumProducte) AS sum_productes_venuts,
        count(distinct idProducte) AS sum_dif_prod,
        sum(beneficiTotal) AS sum_benefici_botiga,
        TRUNC(((sum(beneficiTotal))/(beneficiCadena)*100),2) AS percent_benefici_botiga,
        TRUNC(((sum(beneficiTotal))/(SELECT numTreballadors from BOTIGA WHERE idBotiga=Venta.idBotiga)),2) AS benefici_entre_treballadors
      FROM VENTA
      WHERE EXTRACT(MONTH FROM dataventa)=p_mes AND
        EXTRACT (YEAR FROM dataventa)=p_any
      GROUP BY idBotiga 
      ORDER BY sum_benefici_botiga DESC;
    
    --Si no s'ha produit cap excepci al fer la consulta registrem al LOG i confirmem.  
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
      
    EXCEPTION
    
      --Si l'any indicat no t un format correcte
      WHEN ANY_INCORRECTE THEN
        P_RES:=110001;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
      --Si el mes indicat no t un format correcte
      WHEN MES_INCORRECTE THEN
        p_RES:=110002;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
      --Si no hi ha dades per l'any i mes especificats
      WHEN DADES_NO_DISPONIBLES THEN
        p_RES:=110003;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      
      --Si es dona qualsevol altra excepci  
      WHEN OTHERS THEN
        p_RES:=110004;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);

  END;




  /******************************************
  **       RESUM_MENSUAL_PRODUCTE          **
  ******************************************/

  PROCEDURE RESUM_MENSUAL_PRODUCTE(p_any IN NUMBER, p_mes IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2, p_result OUT SYS_REFCURSOR) IS
    
    --Declarem les variables
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    ANY_INCORRECTE EXCEPTION;
    MES_INCORRECTE EXCEPTION;
    DADES_NO_DISPONIBLES EXCEPTION;
    
    
  BEGIN
    
    --Assignem valors a les variables de LOG
    nomProcediment:='RESUM_MENSUAL_PRODUCTE';
    p_RES:=0;
    paramsProcediment:='P1:'||p_any||' P2:'||p_mes;
    
    --Mirem si el mes i l'any especificats son correctes
    IF p_any NOT BETWEEN 1000 AND 9999 THEN
      RAISE ANY_INCORRECTE;
    END IF;
    IF p_mes NOT BETWEEN 1 AND 12 THEN
      RAISE MES_INCORRECTE;
    END IF;
    
    --Mirem si hi ha dades per aquest perode
    IF HI_HA_DADES(p_any,p_mes)=FALSE THEN
      RAISE DADES_NO_DISPONIBLES;
    END IF;
    
    --Obtenim la consulta en un cursor variable
    open p_result for
      SELECT idProducte AS EAN13_producte, --identificador de producte
        --Nom del producte
        PRODUCTE_PKG.GET_DESC_PRODUCTE(idProducte) AS descripcio_producte,
        --Nombre d'unitats venudes
        sum(sumProducte) AS sum_productes_venuts,
        --benefici net que ha generat el producte
        sum(beneficiTotal) AS sum_benefici_producte,
        --Identificador de la botiga que n'ha venut ms unitats
        BOTIGA_MES_UNITATS(p_any,p_mes,idProducte) AS botiga_mes_ventes,
        --nombre d'unitats venudes per la botiga que n'ha venut ms unitats
        UN_BOTIGA_MES_UNITATS(p_any,p_mes,idProducte) AS unitats_botiga_mes_ventes
      FROM VENTA
      WHERE EXTRACT(MONTH FROM dataventa)=p_mes AND
        EXTRACT (YEAR FROM dataventa)=p_any
      GROUP BY idProducte 
      ORDER BY sum_benefici_producte DESC;
    
    --Si no s'ha produit cap excepci al fer la consulta registrem al LOG i confirmem.  
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;
      
    EXCEPTION
    
      --Si l'any indicat no t un format correcte
      WHEN ANY_INCORRECTE THEN
        P_RES:=111001;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
      --Si el mes indicat no t un format correcte
      WHEN MES_INCORRECTE THEN
        p_RES:=111002;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
      --Si no hi ha dades per l'any i mes especificats
      WHEN DADES_NO_DISPONIBLES THEN
        p_RES:=111003;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      
      --Si es dona qualsevol altra excepci  
      WHEN OTHERS THEN
        p_RES:=111004;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);

  END;




  /******************************************
  **       RESUM_MENSUAL_PER_DIES          **
  ******************************************/

  PROCEDURE RESUM_MENSUAL_PER_DIES(p_any IN NUMBER, p_mes IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2, p_result OUT SYS_REFCURSOR) IS
    
    --Declarem les variables
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    ANY_INCORRECTE EXCEPTION;
    MES_INCORRECTE EXCEPTION;
    DADES_NO_DISPONIBLES EXCEPTION;
    
    BEGIN
    --Assignem valors a les variables de LOG
    nomProcediment:='RESUM_MENSUAL_PER_DIES';
    p_RES:=0;
    paramsProcediment:='P1:'||p_any||' P2:'||p_mes;
    
    --Mirem si el mes i l'any especificats son correctes
    IF p_any NOT BETWEEN 1000 AND 9999 THEN
      RAISE ANY_INCORRECTE;
    END IF;
    IF p_mes NOT BETWEEN 1 AND 12 THEN
      RAISE MES_INCORRECTE;
    END IF;
    
    --Mirem si hi ha dades per aquest perode
    IF HI_HA_DADES(p_any,p_mes)=FALSE THEN
      RAISE DADES_NO_DISPONIBLES;
    END IF;
    
    --Obtenim la consulta en un cursor variable
    open p_result for
      SELECT EXTRACT(DAY FROM dataventa) AS dia_mes, --dia del mes
        --Benefici net obtingut aquell dia
        sum(beneficitotal) AS benefici_net_diari,
        --Identificador producte ms venut
        PRODUCTE_MES_VENUT_DIA(dataventa) AS prod_mes_venut,
        --Nombre d'unitats venudes del producte ms venut
        UN_PRODUCTE_MES_VENUT_DIA(dataventa) AS un_prod_mes_venut,
        --Identificador de la botiga que ms benefici net ha obtingut
        BOTIGA_MES_BENEFICI(dataventa) AS botiga_mes_benefici,
        --valor del benefici net que ha obtingut la botiga
        VAL_BOTIGA_MES_BENEFICI(dataventa) AS valor_botiga_mes_benefici
      FROM VENTA
      WHERE EXTRACT(MONTH FROM dataventa)=p_mes AND
        EXTRACT (YEAR FROM dataventa)=p_any
      GROUP BY dataventa 
      ORDER BY dataventa ASC;
    
    --Si no s'ha produit cap excepci al fer la consulta registrem al LOG i confirmem.  
    logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
    COMMIT;    
    EXCEPTION
    
      --Si l'any indicat no t un format correcte
      WHEN ANY_INCORRECTE THEN
        P_RES:=112001;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
      --Si el mes indicat no t un format correcte
      WHEN MES_INCORRECTE THEN
        p_RES:=112002;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
      --Si no hi ha dades per l'any i mes especificats
      WHEN DADES_NO_DISPONIBLES THEN
        p_RES:=112003;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
      
      --Si es dona qualsevol altra excepci  
      WHEN OTHERS THEN
        p_RES:=112004;
        EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);

  END;




  /******************************************
  **    MOSTRAR_RESUM_MENSUAL_BOTIGA       **
  ******************************************/
  
  PROCEDURE MOSTRAR_RESUM_MENSUAL_BOTIGA(p_any IN NUMBER, p_mes IN NUMBER) IS
    p_RES INT;
    p_ERRDESC VARCHAR2(100);
    p_result SYS_REFCURSOR;
    idBotiga INT;
    sum_productes_venuts INT;
    sum_dif_prod NUMBER;
    sum_benefici_botiga NUMBER;
    percent_benefici_botiga NUMBER;
    benefici_entre_treballadors NUMBER;
  BEGIN
    ---Permetem que la sortida DBMS mostri ms carcters que per defecte.
    DBMS_OUTPUT.ENABLE(1000000);
    dbms_output.put_line('RESUM MENSUAL PER BOTIGUES ANY '||p_any||' MES '||p_mes);
    dbms_output.put_line('-------------------------------------------');
    CONSULTES_PKG.RESUM_MENSUAL_BOTIGA(p_any,p_mes,p_RES,p_ERRDESC,p_result);
    LOOP
      FETCH p_result INTO idBotiga,sum_productes_venuts,sum_dif_prod,sum_benefici_botiga,percent_benefici_botiga,benefici_entre_treballadors;
      EXIT WHEN p_result%NOTFOUND;
      dbms_output.put_line('BOTIGA: '||idBotiga||chr(13)||
        ' PRODUCTES VENUTS: '||sum_productes_venuts||chr(13)||
        ' PRODUCTES DIFERENTS VENUTS: '||sum_dif_prod||chr(13)||
        ' BENEFICI BOTIGA: '||sum_benefici_botiga||''||chr(13)||
        ' PERCENTATGE BENEFICI BOTIGA: '||percent_benefici_botiga||'%'||chr(13)||
        ' BENEFICI PER TREBALLADOR: '||benefici_entre_treballadors||''||chr(13)||chr(13));
    END LOOP;
    CLOSE p_result;
    
  EXCEPTION
    WHEN OTHERS THEN
    dbms_output.put_line('ERROR MOSTRANT LES DADES');
    dbms_output.put_line('');
    
  END;




  /******************************************
  **    MOSTRAR_RESUM_MENSUAL_PRODUCTE     **
  ******************************************/
  
  PROCEDURE MOSTRAR_RESUM_MENSUAL_PRODUCTE(p_any IN NUMBER, p_mes IN NUMBER) IS
    p_RES INT;
    p_ERRDESC VARCHAR2(100);
    p_result SYS_REFCURSOR;
    idProducte INT;
    desProducte Producte.descripcio%TYPE;
    uVenudes INT;
    benefici NUMBER;
    idBotiga NUMBER;
    uVenudesBotiga NUMBER;
    
  BEGIN
    ---Permetem que la sortida DBMS mostri ms carcters que per defecte.
    DBMS_OUTPUT.ENABLE(1000000);
    dbms_output.put_line('RESUM MENSUAL PER PRODUCTES ANY '||p_any||' MES '||p_mes);
    dbms_output.put_line('-------------------------------------------');
    CONSULTES_PKG.RESUM_MENSUAL_PRODUCTE(p_any,p_mes,p_RES,p_ERRDESC,p_result);
    LOOP
      FETCH p_result INTO idProducte, desProducte, uVenudes, benefici, idBotiga, uVenudesBotiga;
      EXIT WHEN p_result%NOTFOUND;
      dbms_output.put_line('PRODUCTE: '||idProducte||chr(13)||
        ' DESCRIPCI: '||desProducte||chr(13)||
        ' UNITATS VENUDES: '||uVenudes||chr(13)||
        ' BENEFICI GENERAT: '||benefici||''||chr(13)||
        ' BOTIGA QUE HA VENUT MS UNITATS: '||idBotiga||chr(13)||
        ' UNITATS VENUDES PER LA BOTIGA: '||uVenudesBotiga||chr(13)||chr(13));
    END LOOP;
    CLOSE p_result;
    
  EXCEPTION
    WHEN OTHERS THEN
      dbms_output.put_line('ERROR MOSTRANT LES DADES');
      dbms_output.put_line('');
      --Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
      --dbms_output.put_line(SQLCODE||': '||SQLERRM); 
  END;




  /******************************************
  **    MOSTRAR_RESUM_MENSUAL_PER_DIES     **
  ******************************************/
  
  PROCEDURE MOSTRAR_RESUM_MENSUAL_PER_DIES(p_any IN NUMBER, p_mes IN NUMBER) IS
    p_RES INT;
    p_ERRDESC VARCHAR2(100);
    p_result SYS_REFCURSOR;
    idDia INT;
    benefici NUMBER;
    prodMesVenut INT;
    unProdMesVenut NUMBER;
    botigaMesBenefici NUMBER;
    valorBotigaMesBenefici NUMBER;
    
  BEGIN
    ---Permetem que la sortida DBMS mostri ms carcters que per defecte.
    DBMS_OUTPUT.ENABLE(1000000);
    dbms_output.put_line('RESUM MENSUAL PER DIES ANY '||p_any||' MES '||p_mes);
    dbms_output.put_line('-------------------------------------------');
    CONSULTES_PKG.RESUM_MENSUAL_PER_DIES(p_any,p_mes,p_RES,p_ERRDESC,p_result);
    LOOP
      FETCH p_result INTO idDia, benefici, prodMesVenut, unProdMesVenut, botigaMesBenefici, valorBotigaMesBenefici;
      EXIT WHEN p_result%NOTFOUND;
      dbms_output.put_line('DIA: '||idDia||chr(13)||
        ' BENEFICI TOTAL CADENA: '||benefici||''||chr(13)||
        ' PRODUCTE MS VENUT: '||prodMesVenut||chr(13)||
        ' UNITATS VENUDES: '||unProdMesVenut||chr(13)||
        ' BOTIGA MS BENEFICI OBTINGUT: '||botigaMesBenefici||chr(13)||
        ' VALOR BENEFICI: '||valorBotigaMesBenefici||''||chr(13)||chr(13));
    END LOOP;
    CLOSE p_result;
    
  EXCEPTION
    WHEN OTHERS THEN
      dbms_output.put_line('ERROR MOSTRANT LES DADES');
      dbms_output.put_line('');
      --Treiem per pantalla l'error que ha generat l'excepci per si volem depurar
      --dbms_output.put_line(SQLCODE||': '||SQLERRM); 
  END;




  /******************************************
  **            HI_HA_DADES                **
  ******************************************/
  
  FUNCTION HI_HA_DADES(p_any NUMBER, p_mes NUMBER) RETURN BOOLEAN IS
    resultat BOOLEAN;
    ccount INT;
  BEGIN
    select count(*) into ccount from venta where EXTRACT(MONTH FROM dataventa)=p_mes AND EXTRACT (YEAR FROM dataventa)=p_any;
    if ccount > 0 THEN
      return TRUE;
    else
      return FALSE;
    END IF;
  END;




  /******************************************
  **       BOTIGA_MES_UNITATS              **
  ******************************************/
  
  FUNCTION BOTIGA_MES_UNITATS (p_any NUMBER, p_mes NUMBER, p_prod NUMBER) RETURN NUMBER IS
    CURSOR ccursor IS SELECT v.idBotiga FROM Venta v WHERE v.idProducte=p_prod AND
      EXTRACT(MONTH FROM dataventa)=p_mes AND EXTRACT (YEAR FROM dataventa)=p_any
      GROUP BY idBotiga ORDER BY sum(sumProducte) DESC;
    idBot NUMBER;
  BEGIN
    OPEN ccursor;
      FETCH ccursor INTO idBot;
    CLOSE ccursor;
    RETURN idBot;
  END;




  /******************************************
  **       UN_BOTIGA_MES_UNITATS           **
  ******************************************/
  
  FUNCTION UN_BOTIGA_MES_UNITATS (p_any NUMBER, p_mes NUMBER, p_prod NUMBER) RETURN NUMBER IS
    CURSOR ccursor IS SELECT SUM(v.sumProducte) FROM Venta v WHERE v.idProducte=p_prod AND
      EXTRACT(MONTH FROM dataventa)=p_mes AND EXTRACT (YEAR FROM dataventa)=p_any
      GROUP BY idBotiga ORDER BY sum(sumProducte) DESC;
    sumProd NUMBER;
  BEGIN
    OPEN ccursor;
      FETCH ccursor INTO sumProd;
    CLOSE ccursor;
    RETURN sumProd;
  END;




  /******************************************
  **       PRODUCTE_MES_VENUT_DIA          **
  ******************************************/
  
  FUNCTION PRODUCTE_MES_VENUT_DIA (p_data DATE) RETURN NUMBER IS
    CURSOR ccursor IS SELECT idProducte FROM Venta v WHERE dataventa=p_data
      GROUP BY idProducte ORDER BY sum(sumProducte) DESC;
    idProd NUMBER;
  BEGIN
    OPEN ccursor;
      FETCH ccursor INTO idProd;
    CLOSE ccursor;
    RETURN idProd;
  END;




  /******************************************
  **      UN_PRODUCTE_MES_VENUT_DIA        **
  ******************************************/
  
  FUNCTION UN_PRODUCTE_MES_VENUT_DIA (p_data DATE) RETURN NUMBER IS
    CURSOR ccursor IS SELECT sum(sumProducte) FROM Venta v WHERE dataventa=p_data
      GROUP BY idProducte ORDER BY sum(sumProducte) DESC;
    sumProd NUMBER;
  BEGIN
    OPEN ccursor;
      FETCH ccursor INTO sumProd;
    CLOSE ccursor;
    RETURN sumProd;
  END;




  /******************************************
  **          BOTIGA_MES_BENEFICI          **
  ******************************************/
  
  FUNCTION BOTIGA_MES_BENEFICI (p_data DATE) RETURN NUMBER IS
    CURSOR ccursor IS SELECT idBotiga FROM Venta v WHERE dataventa=p_data
      GROUP BY idBotiga ORDER BY sum(beneficitotal) DESC;
    idBot NUMBER;
  BEGIN
    OPEN ccursor;
      FETCH ccursor INTO idBot;
    CLOSE ccursor;
    RETURN idBot;
  END;




  /******************************************
  **       VAL_BOTIGA_MES_BENEFICI         **
  ******************************************/
  
  FUNCTION VAL_BOTIGA_MES_BENEFICI (p_data DATE) RETURN NUMBER IS
    CURSOR ccursor IS SELECT sum(beneficitotal) FROM Venta v WHERE dataventa=p_data
      GROUP BY idBotiga ORDER BY sum(beneficitotal) DESC;
    sumBot NUMBER;
  BEGIN
    OPEN ccursor;
      FETCH ccursor INTO sumBot;
    CLOSE ccursor;
    RETURN sumBot;
  END;
  
END;