/**

**/

CREATE OR REPLACE PACKAGE ESTADISTIC_PKG AS
  
  --Procediment que retorna un cursor amb les estadstiques de l'any especificat.
  PROCEDURE EST_ANUAL(p_any IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2 , p_result OUT SYS_REFCURSOR);

  --Procediment que retorna un cursor amb les estadstiques histriques.
  PROCEDURE EST_HISTORIC(p_RES OUT int, p_ERRDESC OUT VARCHAR2 , p_result OUT SYS_REFCURSOR);
  
  --Procediment que mostra l'estadstica d'un any p_any per pantalla, si el valor d'entrada  sun 0 es mostra el valor histric
  PROCEDURE MOSTRAR_ESTADISTICA (p_any IN NUMBER);
  

  --Procediment que inicialitza o actualitza els valors estadstics a partir de la taula de fets
  PROCEDURE INICIALITZAR_ESTADISTIC;

  --Procediment que actualitza totes les taules estadstiques parcials.
  PROCEDURE ACTUALITZAR_EST_PARCIAL (p_any IN NUMBER, p_dia IN NUMBER, p_hora IN NUMBER, p_Benefici IN NUMBER, p_unitats IN NUMBER,
    p_botiga IN NUMBER, p_ciutat IN NUMBER, p_producte NUMBER, p_virtual IN NUMBER);
  
  --Procediment que actualitza totes les estadstiques de p_any sumant-hi tamb el benefici p_benefici
  PROCEDURE ACTUALITZAR_EST_GLOBAL (p_any IN NUMBER);
  
  --Procediment que suma el benefici p_valor a la botiga p_botiga per a l'any p_any
  --a la taula parcial EST_BOTIGA
  PROCEDURE ACTUALITZAR_EST_BOTIGA (p_any IN NUMBER, p_botiga IN NUMBER, p_valor IN NUMBER);
 
  --Procediment que suma el nombre de productes venuts p_valor per al producte p_producte
  --i per a l'any p_any a la taula parcial EST_PRODUCTE 
  PROCEDURE ACTUALITZAR_EST_PRODUCTE (p_any IN NUMBER, p_producte IN NUMBER, p_valor IN NUMBER);
  
  --Procediment que suma el nombre de productes venuts p_valor per a l'hora p_hora
  --i per a l'any p_any a la taula parcial EST_HORA
  PROCEDURE ACTUALITZAR_EST_HORA (p_any IN NUMBER, p_hora IN NUMBER, p_valor IN NUMBER);
  
  --Procediment que suma el nombre de productes venuts p_valor per al dia del mes
  --p_dia i per a l'any p_any a la taula parcial EST_DIA
  PROCEDURE ACTUALITZAR_EST_DIA (p_any IN NUMBER, p_dia IN NUMBER, p_valor IN NUMBER);
  
  --Procediment que suma el benefici p_valor per a la ciutat p_ciutat per a l'any
  --p_any a la taula parcial EST_CIUTAT
  PROCEDURE ACTUALITZAR_EST_CIUTAT (p_any IN NUMBER, p_ciutat IN NUMBER, p_valor IN NUMBER);
  
  --Procediment que suma el benefici virtual i total en un any segons si s un benefici
  --virtual (p_virtual=1) i el valor del benefici p_valor a la taula EST_VIRTUAL  
  PROCEDURE ACTUALITZAR_EST_VIRTUAL (p_any IN NUMBER, p_virtual IN NUMBER, p_valor IN NUMBER);

  --Funci que suma al benefici total per a l'any p_any el valor p_valor a la taula
  --global ESTADISTICA  
  PROCEDURE ACTUALITZAR_BENEFICI_TOTAL (p_any NUMBER);
  
  --Funci que actualitza les dades relatives a la botiga amb ms beneficisper a
  --l'any p_any a la taula global ESTADISTICA
  PROCEDURE ACTUALITZAR_GLOBAL_BOTIGA (p_any NUMBER);
  
  --Funci que actualitza les dades relatives al producte amb ms unitats venudes
  --per a l'any p_any a la taula ESTADISTICA
  PROCEDURE ACTUALITZAR_GLOBAL_PRODUCTE (p_any NUMBER);
  
  --Funci que actualitza les dades relatives a les hores del dia amb ms i menys
  --ventes per a l'any p_any dins la taula ESTADISTICA
  PROCEDURE ACTUALITZAR_GLOBAL_HORA (p_any NUMBER);
  
  --Funci que actualitza les dades relatives als dies del mes amb ms i menys ventes
  --per a l'ant p_any dins la taula ESTADISTICA
  PROCEDURE ACTUALITZAR_GLOBAL_DIA (p_any NUMBER);
  
  --Funci que actualitza les dades relatives a la ciutat amb ms beneficis per a
  --l'any p_any dins la taula ESTADISTICA
  PROCEDURE ACTUALITZAR_GLOBAL_CIUTAT (p_any NUMBER);
  
  --Funci que actualitza les dades relatives al % de beneficis virtuals per a l'any
  --p_any disn la taula ESTADISTICA
  PROCEDURE ACTUALITZAR_GLOBAL_VIRTUAL (p_any NUMBER);

END ESTADISTIC_PKG;
/

CREATE OR REPLACE PACKAGE BODY ESTADISTIC_PKG AS

  /******************************************
  **              EST_ANUAL                **
  ******************************************/
  PROCEDURE EST_ANUAL(p_any IN NUMBER, p_RES OUT int, p_ERRDESC OUT VARCHAR2 , p_result OUT SYS_REFCURSOR) IS
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    existeix INT;
    ANY_INCORRECTE EXCEPTION;
    DADES_NO_DISPONIBLES EXCEPTION;
  BEGIN
  
    --Assignem valors a les variables de LOG
    nomProcediment:='EST_ANUAL';
      p_RES:=0;
      paramsProcediment:='P1:'||p_any;
    
    --Mirem si l'any s correcte
    IF p_any NOT BETWEEN 1000 AND 9999 THEN
        RAISE ANY_INCORRECTE;
      END IF;
    --Mirem si hi ha dades per aquest any
    SELECT COUNT(*) INTO existeix FROM ESTADISTICA WHERE idAny=p_any;
    IF existeix=0 THEN
      RAISE DADES_NO_DISPONIBLES;
    END IF;
    
    --Obtenim la consulta en un cursor variable
    open p_result for
      select * from ESTADISTICA where idAny=p_any;
    
      --Si no s'ha produit cap excepci al fer la consulta registrem al LOG i confirmem.  
      logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
      COMMIT;	
	
  EXCEPTION
    --Si l'any indicat no t un format correcte
    WHEN ANY_INCORRECTE THEN
      P_RES:=113001;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
        
    --Si no hi ha dades per l'any especificat
    WHEN DADES_NO_DISPONIBLES THEN
      p_RES:=113002;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
    --Si es dona qualsevol altra excepci  
    WHEN OTHERS THEN
      p_RES:=113003;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
  
  END;
  
  
  
  /******************************************
  **              EST_HISTORIC             **
  ******************************************/
  PROCEDURE EST_HISTORIC(p_RES OUT int, p_ERRDESC OUT VARCHAR2 , p_result OUT SYS_REFCURSOR) IS
    nomProcediment VARCHAR2(30);
    paramsProcediment VARCHAR2(500);
    logREG INT;
    existeix INT;
    DADES_NO_DISPONIBLES EXCEPTION;
  BEGIN
  
    --Assignem valors a les variables de LOG
    nomProcediment:='EST_ANUAL';
      p_RES:=0;
      paramsProcediment:=NULL;
    
    --Mirem si hi ha dades a l'histric
    SELECT COUNT(*) INTO existeix FROM ESTADISTICA;
    IF existeix=0 THEN
      RAISE DADES_NO_DISPONIBLES;
    END IF;
    
    --Obtenim la consulta en un cursor variable
    open p_result for
      select * from ESTADISTICA where idAny=0;
    
      --Si no s'ha produit cap excepci al fer la consulta registrem al LOG i confirmem.  
      logREG:=LOG_PKG.registrar_log(nomProcediment,paramsProcediment,p_RES);
      COMMIT;	
	
  EXCEPTION
        
    --Si no hi ha dades per l'any
    WHEN DADES_NO_DISPONIBLES THEN
      p_RES:=114001;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
    
    --Si es dona qualsevol altra excepci  
    WHEN OTHERS THEN
      p_RES:=114002;
      EXCEPTION_PKG.GEST_EX(p_RES,nomProcediment,paramsProcediment,p_ERRDESC);
  
  END;
  
  
  
  /******************************************
  **         MOSTRAR_ESTADISTICA           **
  ******************************************/
  PROCEDURE MOSTRAR_ESTADISTICA (p_any IN NUMBER) IS
    p_RES INT;
    p_ERRDESC VARCHAR2(100);
    p_result SYS_REFCURSOR;
    eAny INT;
    eBenefici NUMBER;
    eBotigaMax INT;
    eBotigaMB NUMBER;
    eProducteMax INT;
    eProducteMV INT;
    eHoraMax INT;
    eHoraMaxV INT;
    eHoraMin INT;
    eHoraMinV INT;
    eDiaMax INT;
    eDiaMaxV INT;
    eDiaMin INT;
    eDiaMinV INT;
    eCiutatMax INT;
    eCiutatMaxB NUMBER;
    ePercentV NUMBER;
  BEGIN
    ---Permetem que la sortida DBMS mostri ms carcters que per defecte.
    DBMS_OUTPUT.ENABLE(1000000);
    IF (p_any!=0) THEN
      dbms_output.put_line('ESTADSTIQUES ANY '||p_any);
      dbms_output.put_line('-------------------------------------------');
      EST_ANUAL(p_any,p_RES,p_ERRDESC,p_result);
    ELSE
      dbms_output.put_line('ESTADSTIQUES HISTRIQUES ');
      dbms_output.put_line('-------------------------------------------');
      EST_HISTORIC(p_RES,p_ERRDESC,p_result);
    END IF;
    
    FETCH p_result INTO eAny,eBenefici,eBotigaMax,eBotigaMB,eProducteMax,eProducteMV,eHoraMax,eHoraMaxV,eHoraMin,eHoraMinV,
      eDiaMax,eDiaMaxV,eDiaMin,eDiaMinV,eCiutatMax,eCiutatMaxB,ePercentV;
    dbms_output.put_line(' BENEFICI NET TOTAL DE LA CADENA: '||eBenefici||''||chr(13)||
        ' BOTIGA AMB MS BENEFICIS: '||eBotigaMax||chr(13)||
        ' VALOR DELS BENEFICIS: '||eBotigaMB||''||chr(13)||
        ' PRODUCTE MS VENUT: '||eProducteMax||chr(13)||
        ' NOMBRE UNITATS VENUDES: '||eProducteMV||chr(13)||
        ' HORA DEL DIA AMB MS PRODUCTES VENUTS: '||eHoraMax||chr(13)||
        ' NOMBRE DE PRODUCTES VENUTS: '||eHoraMaxV||chr(13)||
        ' HORA DEL DIA AMB MENYS PRODUCTES VENUTS: '||eHoraMin||chr(13)||
        ' NOMBRE DE PRODUCTES VENUTS: '||eHoraMinV||chr(13)||
        ' DIA DEL MES AMB MS PRODUCTES VENUTS: '||eDiaMax||chr(13)||
        ' NOMBRE DE PRODUCTES VENUTS: '||eDiaMaxV||chr(13)||
        ' DIA DEL MES AMB MENYS PRODUCTES VENUTS: '||eDiaMin||chr(13)||
        ' NOMBRE DE PRODUCTES VENUTS: '||eDiaMinV||chr(13)||
        ' CIUTAT AMB MS BENEFICIS: '||eCiutatMax||chr(13)||
        ' VALOR DELS BENEFICIS: '||eciutatMaxB||''||chr(13)||
        ' % BENEFICI BOTIGUES VIRTUALS: '||ePercentV||'%'||chr(13)||chr(13)); 
    CLOSE p_result;
    
  EXCEPTION
    WHEN OTHERS THEN
    dbms_output.put_line('ERROR MOSTRANT LES DADES');
    dbms_output.put_line('');
  END;
  
  
  
  /******************************************
  **       INICIALITZAR_ESTADSTIC         **
  ******************************************/  
  PROCEDURE INICIALITZAR_ESTADISTIC IS
    CURSOR cventes IS SELECT VENTA.idBotiga,idProducte,dataventa,horaventa,sumproducte,beneficitotal,BOTIGA.esVirtual,BOTIGA.idCiutat  FROM VENTA, BOTIGA WHERE VENTA.idBotiga=BOTIGA.idBotiga;
    CURSOR cAnys IS SELECT DISTINCT(EXTRACT(YEAR FROM dataVenta)) FROM VENTA;
    botiga INT;
    producte INT;
    dataV DATE;
    horaV INT;
    sumProd NUMBER;
    benefici NUMBER;
    ciutat NUMBER;
    isVirtual NUMBER;
    p_any NUMBER;
  BEGIN
    --Esborrem totes les dades estadstiques que tinguem desades
    delete from EST_BOTIGA;
    delete from est_ciutat;
    delete from est_dia;
    delete from est_hora;
    delete from est_producte;
    delete from est_virtual;
    delete from estadistica;
    
    --Recorrem la taula VENTA per conformar les dades estadstiques parcials
    OPEN cventes;
    LOOP
      FETCH cventes INTO botiga,producte,dataV,horaV,sumProd,benefici,isVirtual,ciutat;
      EXIT WHEN cventes%NOTFOUND;
        ACTUALITZAR_EST_PARCIAL(EXTRACT(YEAR FROM dataV),EXTRACT(DAY FROM dataV),horaV,
          benefici,sumProd,botiga,ciutat,producte,isVirtual);
    END LOOP;  
    CLOSE cventes;
    
    --Actualitzem les estadstiques globals
    OPEN cAnys;
    LOOP
      FETCH cAnys INTO p_any;
      EXIT WHEN cAnys%NOTFOUND;
      ACTUALITZAR_EST_GLOBAL(p_any);
    END LOOP;
    CLOSE cAnys;
   
  END;




  PROCEDURE ACTUALITZAR_EST_PARCIAL (p_any IN NUMBER, p_dia IN NUMBER, p_hora IN NUMBER, p_Benefici IN NUMBER, p_unitats IN NUMBER,
    p_botiga IN NUMBER, p_ciutat IN NUMBER, p_producte NUMBER, p_virtual IN NUMBER) IS
  BEGIN
    ACTUALITZAR_EST_BOTIGA(p_any,p_botiga,p_benefici);
    ACTUALITZAR_EST_PRODUCTE(p_any,p_producte,p_unitats);
    ACTUALITZAR_EST_HORA(p_any,p_hora,p_unitats);
    ACTUALITZAR_EST_DIA(p_any,p_dia,p_unitats);
    ACTUALITZAR_EST_CIUTAT(p_any,p_ciutat,p_benefici);
    ACTUALITZAR_EST_VIRTUAL(p_any,p_virtual,p_benefici);
  END;


  PROCEDURE ACTUALITZAR_EST_GLOBAL (p_any IN NUMBER) IS
  existeix INT;
  BEGIN
	--Si no hi ha estadstics registrats per aquest any eliminem l'estadstica general d'aquest any.
	SELECT count(*) INTO existeix FROM EST_VIRTUAL WHERE idAny=p_any;
	IF existeix=0 THEN
	  DELETE FROM ESTADISTICA WHERE idAny=p_any;
	  SELECT count(*) INTO existeix FROM ESTADISTICA;
	  IF existeix=1 THEN
		DELETE FROM ESTADISTICA;
	  END IF;
	ELSE
	  ACTUALITZAR_BENEFICI_TOTAL(p_any);
      ACTUALITZAR_GLOBAL_BOTIGA(p_any);
      ACTUALITZAR_GLOBAL_PRODUCTE(p_any);
      ACTUALITZAR_GLOBAL_HORA(p_any);
      ACTUALITZAR_GLOBAL_DIA(p_any);
      ACTUALITZAR_GLOBAL_CIUTAT(p_any);
      ACTUALITZAR_GLOBAL_VIRTUAL(p_any);
	END IF;
  END;


  /******************************************
  **        ACTUALITZAR_EST_BOTIGA         **
  ******************************************/
  PROCEDURE ACTUALITZAR_EST_BOTIGA (p_any IN NUMBER, p_botiga IN NUMBER, p_valor IN NUMBER) IS
    existeix INT;
    benefici NUMBER;
  BEGIN
    --Actualitzem la taula EST_PRODUCTE
    SELECT COUNT(*) INTO existeix FROM EST_BOTIGA WHERE idAny=p_any AND idBotiga=p_botiga;
    IF existeix=0 THEN
      INSERT INTO EST_BOTIGA VALUES (p_any, p_botiga, p_valor);
    ELSE
      SELECT beneficiBotiga INTO benefici FROM EST_BOTIGA WHERE idAny=p_any AND idBotiga=p_botiga;
      IF benefici+p_valor=0 THEN
        DELETE FROM EST_BOTIGA WHERE idAny=p_any AND idBotiga=p_botiga;
      ELSE
        UPDATE EST_BOTIGA SET beneficiBotiga=beneficiBotiga+p_valor WHERE idAny=p_any AND idBotiga=p_botiga;
      END IF;
    END IF;
  END;  
  
  
  
  /******************************************
  **       ACTUALITZAR_EST_PRODUCTE        **
  ******************************************/
  PROCEDURE ACTUALITZAR_EST_PRODUCTE (p_any IN NUMBER, p_producte IN NUMBER, p_valor IN NUMBER) IS
    existeix INT;
    venuts NUMBER;
  BEGIN
    --Actualitzem la taula EST_PRODUCTE
    SELECT COUNT(*) INTO existeix FROM EST_PRODUCTE WHERE idAny=p_any AND idProducte=p_producte;
    IF existeix=0 THEN
      INSERT INTO EST_PRODUCTE VALUES (p_any, p_producte, p_valor);
    ELSE
      SELECT numvenuts INTO venuts FROM EST_PRODUCTE WHERE idAny=p_any AND idProducte=p_producte;
      IF venuts+p_valor=0 THEN
        DELETE FROM EST_PRODUCTE WHERE idAny=p_any AND idProducte=p_producte;
      ELSE
        UPDATE EST_PRODUCTE SET numVenuts=numVenuts+p_valor WHERE idAny=p_any AND idProducte=p_producte;
      END IF;
    END IF;

  END;
  
  
  
  /******************************************
  **         ACTUALITZAR_EST_HORA          **
  ******************************************/    
  PROCEDURE ACTUALITZAR_EST_HORA (p_any IN NUMBER, p_hora IN NUMBER, p_valor IN NUMBER) IS
    existeix INT;
    venuts NUMBER;
  BEGIN
    SELECT COUNT(*) INTO existeix FROM EST_HORA WHERE idAny=p_any AND idHora=p_hora;
    IF existeix=0 THEN
      INSERT INTO EST_HORA VALUES (p_any, p_hora, p_valor);
    ELSE
      SELECT n_prod_venuts INTO venuts FROM EST_HORA WHERE idAny=p_any AND idHora=p_hora;
      IF venuts+p_valor=0 THEN
        DELETE FROM EST_HORA WHERE idAny=p_any AND idHora=p_hora;
      ELSE
        UPDATE EST_HORA SET n_prod_venuts=n_prod_venuts+p_valor WHERE idAny=p_any AND idHora=p_hora;
      END IF;
    END IF;
    
  END;




  /******************************************
  **         ACTUALITZAR_EST_DIA           **
  ******************************************/  
  PROCEDURE ACTUALITZAR_EST_DIA (p_any IN NUMBER, p_dia IN NUMBER, p_valor IN NUMBER) IS
    existeix INT;
    venuts NUMBER;
  BEGIN
    SELECT COUNT(*) INTO existeix FROM EST_DIA WHERE idAny=p_any AND idDia=p_dia;
    IF existeix=0 THEN
      INSERT INTO EST_DIA VALUES (p_any, p_dia, p_valor);
    ELSE
      SELECT n_prod_venuts INTO venuts FROM EST_DIA WHERE idAny=p_any AND idDia=p_dia;
      IF venuts+p_valor=0 THEN
        DELETE FROM EST_DIA WHERE idAny=p_any AND idDia=p_dia;
      ELSE
        UPDATE EST_DIA SET n_prod_venuts=n_prod_venuts+p_valor WHERE idAny=p_any AND idDia=p_dia;
      END IF;
    END IF;
  END;
  
  
  
  /******************************************
  **        ACTUALITZAR_EST_CIUTAT         **
  ******************************************/
  PROCEDURE ACTUALITZAR_EST_CIUTAT (p_any IN NUMBER, p_ciutat IN NUMBER, p_valor IN NUMBER) IS
    existeix INT;
	benefici NUMBER;
  BEGIN
    SELECT COUNT(*) INTO existeix FROM EST_CIUTAT WHERE idAny=p_any AND idCiutat=p_ciutat;
    IF existeix=0 THEN
      INSERT INTO EST_CIUTAT VALUES (p_any, p_ciutat, p_valor);
    ELSE
      SELECT beneficis INTO benefici FROM EST_CIUTAT WHERE idAny=p_any AND idCiutat=p_ciutat;
      IF benefici+p_valor=0 THEN
        DELETE FROM EST_CIUTAT WHERE idAny=p_any AND idCiutat=p_ciutat;
      ELSE
        UPDATE EST_CIUTAT SET beneficis=beneficis+p_valor WHERE idAny=p_any AND idCiutat=p_ciutat;
      END IF;
    END IF;


  END;  
  
  
  
  /******************************************
  **        ACTUALITZAR_EST_VIRTUAL        **
  ******************************************/ 
  PROCEDURE ACTUALITZAR_EST_VIRTUAL (p_any IN NUMBER, p_virtual IN NUMBER, p_valor IN NUMBER) IS
    percentV NUMBER;
    beneficiV NUMBER;
    beneficiT NUMBER;
    existeix NUMBER;
   BEGIN
    --dbms_output.put_line('Any:'||p_any||' Virtual:'||p_virtual||' Benefici:'||p_valor);
    SELECT count(*) INTO existeix FROM EST_VIRTUAL WHERE idAny=p_any;
    IF existeix=0 THEN
      --dbms_output.put_line('No hi ha estadstica per aquest any');
      IF p_virtual=1 THEN
        --dbms_output.put_line('s virtual');
        INSERT INTO EST_VIRTUAL (idAny, beneficiVirtual, beneficiTotal) VALUES (p_any,p_valor,p_valor);
      ELSE
        --dbms_output.put_line('No s virtual');
        INSERT INTO EST_VIRTUAL (idAny, beneficiVirtual, beneficiTotal) VALUES (p_any,0,p_valor);
      END IF;
    ELSE
      SELECT beneficiTotal, beneficiVirtual INTO beneficiT, beneficiV FROM EST_VIRTUAL WHERE idAny=p_any;
      IF p_virtual=1 THEN
        IF (beneficiT+p_valor=0)AND(beneficiV+p_valor=0) THEN
          DELETE FROM EST_VIRTUAL WHERE idAny=p_any;
        ELSE
          UPDATE EST_VIRTUAL SET beneficiVirtual=beneficiVirtual+p_valor, beneficiTotal=beneficiTotal+p_valor WHERE idAny=p_any;
        END IF;
      ELSE
        IF (beneficiT+p_valor=0)AND(beneficiV=0) THEN
          DELETE FROM EST_VIRTUAL WHERE idAny=p_any;
        ELSE
          UPDATE EST_VIRTUAL SET beneficiTotal=beneficiTotal+p_valor WHERE idAny=p_any;
        END IF;
      END IF;
    END IF;
    
   END;



  /******************************************

  **       ACTUALITZAR_BENEFICI_TOTAL      **
  ******************************************/
  PROCEDURE ACTUALITZAR_BENEFICI_TOTAL (p_any IN NUMBER) IS
    existeix NUMBER;
  BEGIN

    SELECT count(*) INTO existeix FROM ESTADISTICA WHERE idAny=p_any;
    --Actualitzem l'estadstica per a l'any p_any
    IF existeix=0 THEN
      INSERT INTO ESTADISTICA(idAny,sumBenefici) VALUES (p_any,(SELECT beneficiTotal FROM EST_VIRTUAL WHERE idAny=p_any));
    ELSE
      UPDATE ESTADISTICA SET sumBenefici=(SELECT beneficiTotal FROM EST_VIRTUAL WHERE idAny=p_any) WHERE idAny=p_any;
    END IF;
    
    SELECT count(*) INTO existeix FROM ESTADISTICA WHERE idAny=0;
    --Actualitzem l'estadstica per al global
    IF existeix=0 THEN
      INSERT INTO ESTADISTICA(idAny,sumBenefici) VALUES (0,(SELECT sum(beneficiTotal) FROM EST_VIRTUAL));
    ELSE
      UPDATE ESTADISTICA SET sumBenefici=(SELECT sum(beneficiTotal) FROM EST_VIRTUAL) WHERE idAny=0;
    END IF;
  END;
  
  
  
  /******************************************
  **       ACTUALITZAR_GLOBAL_BOTIGA       **
  ******************************************/  
  PROCEDURE ACTUALITZAR_GLOBAL_BOTIGA (p_any IN NUMBER) IS
    botiga NUMBER;
    benefici NUMBER;
    CURSOR cEstBotiga IS SELECT idBotiga, beneficiBotiga FROM EST_BOTIGA WHERE idAny=p_any ORDER BY beneficiBotiga DESC;
    CURSOR cEstBotigaG IS SELECT idBotiga, sum(beneficiBotiga) FROM EST_BOTIGA GROUP BY idBotiga ORDER BY sum(beneficiBotiga) DESC;
  BEGIN
    OPEN cEstBotiga;
      FETCH cEstBotiga INTO botiga, benefici;
    CLOSE cEstBotiga;
    
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idBotigaMaxBenefici=botiga , sumBotigaMaxBenefici=benefici WHERE idAny=p_any; 
  
    OPEN cEstBotigaG;
      FETCH cEstBotigaG INTO botiga, benefici;
    CLOSE cEstBotigaG;
    
    --Suposem que l'estadstica global ja est creada.
    UPDATE ESTADISTICA SET idBotigaMaxBenefici=botiga , sumBotigaMaxBenefici=benefici WHERE idAny=0;
  
  END;
  
  
  
  /******************************************
  **       ACTUALITZAR_GLOBAL_PRODUCTE     **
  ******************************************/    
  PROCEDURE ACTUALITZAR_GLOBAL_PRODUCTE (p_any IN NUMBER) IS
    producte NUMBER;
    unitats NUMBER;
    CURSOR cEstProducte IS SELECT idProducte, numVenuts FROM EST_PRODUCTE WHERE idAny=p_any ORDER BY numVenuts DESC;
    CURSOR cEstProducteG IS SELECT idProducte, sum(numVenuts) FROM EST_PRODUCTE GROUP BY idProducte ORDER BY sum(numVenuts) DESC;
  BEGIN
    --Actualitzem l'estadstica per a l'any p_any
    OPEN cEstProducte;
      FETCH cEstProducte INTO producte, unitats;
    CLOSE cEstProducte;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idProducteMaxVendes=producte, sumProducteMaxVendes=unitats WHERE idAny=p_any; 

    --Actualitzem l'estadstic global
    OPEN cEstProducteG;
      FETCH cEstProducteG INTO producte, unitats;
    CLOSE cEstProducteG;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idProducteMaxVendes=producte, sumProducteMaxVendes=unitats WHERE idAny=0;  
  END;



  /******************************************
  **       ACTUALITZAR_GLOBAL_HORA         **
  ******************************************/    

  PROCEDURE ACTUALITZAR_GLOBAL_HORA (p_any IN NUMBER) IS
    hora INT;
    unitats INT;
    CURSOR cEstHoraDesc IS SELECT idHora, n_prod_venuts FROM EST_HORA WHERE idAny=p_any ORDER BY n_prod_venuts DESC;
    CURSOR cEstHoraGDesc IS SELECT idHora, sum(n_prod_venuts) FROM EST_HORA GROUP BY idHora ORDER BY sum(n_prod_venuts) DESC;
    CURSOR cEstHoraAsc IS SELECT idHora, n_prod_venuts FROM EST_HORA WHERE idAny=p_any ORDER BY n_prod_venuts ASC;
    CURSOR cEstHoraGAsc IS SELECT idHora, sum(n_prod_venuts) FROM EST_HORA GROUP BY idHora ORDER BY sum(n_prod_venuts) ASC;
  BEGIN

    --Actualitzem l'estadstica per a l'any p_any
    OPEN cEstHoraDesc;
      FETCH cEstHoraDesc INTO hora, unitats;
    CLOSE cEstHoraDesc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idHoraMaxVendes=hora, sumHoraMaxVendes=unitats WHERE idAny=p_any; 

    OPEN cEstHoraAsc;
      FETCH cEstHoraAsc INTO hora, unitats;
    CLOSE cEstHoraAsc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idHoraMinVendes=hora, sumHoraMinVendes=unitats WHERE idAny=p_any; 


    --Actualitzem l'estadstic global
    OPEN cEstHoraGDesc;
      FETCH cEstHoraGDesc INTO hora, unitats;
    CLOSE cEstHoraGDesc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idHoraMaxVendes=hora, sumHoraMaxVendes=unitats WHERE idAny=0;  
    
    OPEN cEstHoraGAsc;
      FETCH cEstHoraGAsc INTO hora, unitats;
    CLOSE cEstHoraGAsc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idHoraMinVendes=hora, sumHoraMinVendes=unitats WHERE idAny=0;  
    
  END;  



  /******************************************
  **       ACTUALITZAR_GLOBAL_DIA          **
  ******************************************/    
  PROCEDURE ACTUALITZAR_GLOBAL_DIA (p_any IN NUMBER) IS
    dia INT;
    unitats INT;
    CURSOR cEstDiaDesc IS SELECT idDia, n_prod_venuts FROM EST_DIA WHERE idAny=p_any ORDER BY n_prod_venuts DESC;
    CURSOR cEstDiaGDesc IS SELECT idDia, sum(n_prod_venuts) FROM EST_DIA GROUP BY idDia ORDER BY sum(n_prod_venuts) DESC;
    CURSOR cEstDiaAsc IS SELECT idDia, n_prod_venuts FROM EST_DIA WHERE idAny=p_any ORDER BY n_prod_venuts ASC;
    CURSOR cEstDiaGAsc IS SELECT idDia, sum(n_prod_venuts) FROM EST_DIA GROUP BY idDia ORDER BY sum(n_prod_venuts) ASC;
  BEGIN

    --Actualitzem l'estadstica per a l'any p_any
    OPEN cEstDiaDesc;
      FETCH cEstDiaDesc INTO dia, unitats;
    CLOSE cEstDiaDesc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idDiaMaxVendes=dia, sumDiaMaxVendes=unitats WHERE idAny=p_any; 

    OPEN cEstDiaAsc;
      FETCH cEstDiaAsc INTO dia, unitats;
    CLOSE cEstDiaAsc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idDiaMinVendes=dia, sumDiaMinVendes=unitats WHERE idAny=p_any; 


    --Actualitzem l'estadstic global
    OPEN cEstDiaGDesc;
      FETCH cEstDiaGDesc INTO dia, unitats;
    CLOSE cEstDiaGDesc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idDiaMaxVendes=dia, sumDiaMaxVendes=unitats WHERE idAny=0;  
    
    OPEN cEstDiaGAsc;
      FETCH cEstDiaGAsc INTO dia, unitats;
    CLOSE cEstDiaGAsc;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idDiaMinVendes=dia, sumDiaMinVendes=unitats WHERE idAny=0;  
    
  END;  



  /******************************************
  **       ACTUALITZAR_GLOBAL_CIUTAT       **
  ******************************************/    
  PROCEDURE ACTUALITZAR_GLOBAL_CIUTAT (p_any IN NUMBER) IS
    ciutat INT;
    benefici NUMBER;
    CURSOR cEstCiutat IS SELECT idCiutat, beneficis FROM EST_CIUTAT WHERE idAny=p_any ORDER BY beneficis DESC;
    CURSOR cEstCiutatG IS SELECT idCiutat, sum(beneficis) FROM EST_CIUTAT GROUP BY idCiutat ORDER BY sum(beneficis) DESC;
  BEGIN
    --Actualitzem l'estadstica per a l'any p_any
    OPEN cEstCiutat;
      FETCH cEstCiutat INTO ciutat, benefici;
    CLOSE cEstCiutat;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idCiutatMaxBenefici=ciutat, sumCiutatMaxBenefici=benefici WHERE idAny=p_any; 

    --Actualitzem l'estadstic global
    OPEN cEstCiutatG;
      FETCH cEstCiutatG INTO ciutat, benefici;
    CLOSE cEstCiutatG;
    --Suposem que l'estadstica per aquest any ja est creada.
    UPDATE ESTADISTICA SET idCiutatMaxBenefici=ciutat, sumCiutatMaxBenefici=benefici WHERE idAny=0;
    
  END;




  /******************************************
  **       ACTUALITZAR_GLOBAL_VIRTUAL      **
  ******************************************/    
  PROCEDURE ACTUALITZAR_GLOBAL_VIRTUAL (p_any IN NUMBER) IS
    beneficiT NUMBER;
    beneficiV NUMBER;
    percentatge NUMBER;
  BEGIN
    --Actualitzem l'estadstic per a l'any p_any;
    SELECT beneficiVirtual, beneficiTotal INTO beneficiV, beneficiT FROM EST_VIRTUAL WHERE idAny=p_any;
    IF beneficiV=0 THEN
      percentatge:=0;
    ELSE
      percentatge:=TRUNC((beneficiV/beneficiT*100),2);
    END IF;

    UPDATE ESTADISTICA SET percentBeneficiVirtual=percentatge WHERE idAny=p_any;

    --Actualitzem l'estadstic per al global
    SELECT sum(beneficiVirtual), sum(beneficiTotal) INTO beneficiV, beneficiT FROM EST_VIRTUAL;
    IF beneficiV=0 THEN
      percentatge:=0;
    ELSE
      percentatge:=TRUNC((beneficiV/beneficiT*100),2);
    END IF;    
    UPDATE ESTADISTICA SET percentBeneficiVirtual=percentatge WHERE idAny=0;
    
  END;


  
END;